@@ -4,6 +4,9 @@
 /* */
 #undef VERSION
 
+/* */
+#undef HAVE_OPENMP
+
 /* */
 #undef HAVE_TYPEOF
 
@@ -25,6 +28,12 @@
 /* */
 #undef HAVE_VOLATILE
 
+/* */
+#undef HAVE_RESTRICT
+
+/* */
+#undef HAVE___RESTRICT
+
 /* */
 #undef HAVE_ERRNO
 
@@ -55,15 +64,12 @@
 /* */
 #undef HAVE_STRNCMP
 
-/* */
-#undef HAVE_STRCASECMP
-
-/* */
-#undef HAVE_STRNCASECMP
-
 /* */
 #undef HAVE_STRCSPN
 
+/* */
+#undef HAVE_STRSPN
+
 /* */
 #undef HAVE_STRSTR
 
@@ -82,17 +88,26 @@
 /* */
 #undef HAVE_MEMSET
 
+/* */
+#undef HAVE_MEMMEM
+
 /* */
 #undef HAVE_STRERROR
 
+/* */
+#undef HAVE_SIGFILLSET
+
 /* */
 #undef HAVE_SIGSETJMP
 
 /* */
-#undef HAVE_POW
+#undef HAVE___BUILTIN_ADD_OVERFLOW
 
 /* */
-#undef HAVE_POWF
+#undef DEBUGLEVEL
+
+/* */
+#undef HAVE_CLOCK_GETTIME
 
 /* */
 #undef HAVE_GETHOSTBYNAME
@@ -107,7 +122,10 @@
 #undef SUPPORT_IPV6_SCOPE
 
 /* */
-#undef DEBUGLEVEL
+#undef HAVE_POW
+
+/* */
+#undef HAVE_POWF
 
 /* */
 #undef JS
@@ -189,3 +207,9 @@
 
 /* Tiff by Brain */
 #undef HAVE_TIFF
+
+/* SVG */
+#undef HAVE_SVG
+
+/* Gpm_Event has wdx and wdy */
+#undef HAVE_WDX_WDY
