@@ -5,6 +5,8 @@
 
 #if defined(__BEOS__) || defined(__HAIKU__)
 
+#include "com-defs.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -214,9 +216,7 @@ int get_input_handle(void)
 	if ((inth = start_thr(input_handle_th, NULL, "input_thread")) < 0) {
 		closesocket(ihpipe[0]);
 		closesocket(ihpipe[1]);
-		error("Can't spawn input thread");
-		fatal_tty_exit();
-		exit(4);
+		fatal_exit("Can't spawn input thread");
 	}
 	return h = ihpipe[0];
 }
@@ -253,9 +253,7 @@ int get_output_handle(void)
 	if (start_thr(output_handle_th, NULL, "output_thread") < 0) {
 		closesocket(ohpipe[0]);
 		closesocket(ohpipe[1]);
-		error("Can't spawn output thread");
-		fatal_tty_exit();
-		exit(4);
+		fatal_exit("Can't spawn output thread");
 	}
 	return h = ohpipe[1];
 }*/
