@@ -429,7 +429,7 @@ static unsigned char *p_arse_options(int argc, unsigned char *argv[], struct opt
 			struct option **op;
 			for (op = opt; (options = *op); op++) for (i = 0; options[i].p; i++)
 				if (options[i].rd_cmd && options[i].cmd_name &&
-				    !strcasecmp(cast_const_char options[i].cmd_name, cast_const_char &argv[-1][1])) {
+				    !casestrcmp(cast_uchar options[i].cmd_name, &argv[-1][1])) {
 					if ((e = options[i].rd_cmd(&options[i], &argv, &argc))) {
 						if (e[0]) fprintf(stderr, "Error parsing option %s: %s\n", argv[-1], e);
 						return NULL;
@@ -540,7 +540,7 @@ static void parse_config_file(unsigned char *name, unsigned char *file, struct o
 		f:
 		if (tok) mem_free(tok);
 	}
-	if (err) fprintf(stderr, "\007"), sleep(1);
+	if (err) fprintf(stderr, "\007"), portable_sleep(1000);
 }
 
 static unsigned char *create_config_string(struct option *options)
@@ -658,7 +658,7 @@ static void translate_vms_to_unix(unsigned char **str)
 	unsigned char *n;
 	if (!*str || strchr(cast_const_char *str, '/')) return;
 	n = cast_uchar decc$translate_vms(cast_const_char *str);
-	if (!n || (int)n == -1) return;
+	if (!n || (my_intptr_t)n == -1) return;
 	mem_free(*str);
 	*str = stracpy(n);
 }
@@ -728,7 +728,7 @@ skip_path_conv:;
 	translate_vms_to_unix(&config_dir);
 #endif
 	if (!home) {
-  		int i;
+		int i;
 		home = stracpy(path_to_exe);
 		if (!home) {
 			if (config_dir) mem_free(config_dir);
@@ -752,13 +752,20 @@ skip_path_conv:;
 			add_to_strn(&home_links, cast_uchar "/links");
 		} else {
 			fprintf(stderr, "CONFIG_DIR set to %s. But directory %s doesn't exist.\n\007", config_dir, home_links);
-			sleep(3);
+			portable_sleep(3000);
 			mem_free(home_links);
 			home_links = stracpy(home);
-			add_to_strn(&home_links, cast_uchar ".links");
+			goto add_dot_links;
 		}
 		mem_free(config_dir);
-	} else add_to_strn(&home_links, cast_uchar ".links");
+	} else {
+		add_dot_links:
+#ifdef OPENVMS
+		add_to_strn(&home_links, cast_uchar "links");
+#else
+		add_to_strn(&home_links, cast_uchar ".links");
+#endif
+	}
 	EINTRLOOP(rs, stat(cast_const_char home_links, &st));
 	if (rs) {
 #ifdef HAVE_MKDIR
@@ -830,7 +837,7 @@ void init_home(void)
 	links_home = get_home(&first_use);
 	if (!links_home) {
 		fprintf(stderr, "Unable to find or create links config directory. Please check, that you have $HOME variable set correctly and that you have write permission to your home directory.\n\007");
-		sleep(3);
+		portable_sleep(3000);
 		return;
 	}
 }
@@ -846,6 +853,7 @@ static int write_config_data(unsigned char *prefix, unsigned char *name, struct
 	config_file = stracpy(prefix);
 	if (!config_file) {
 		mem_free(c);
+		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", TEXT_(T_HOME_DIRECTORY_INACCESSIBLE), NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
 		return -1;
 	}
 	add_to_strn(&config_file, name);
@@ -989,8 +997,8 @@ static unsigned char *lang_rd(struct option *o, unsigned char *c)
 	int i;
 	unsigned char *tok = get_token(&c);
 	if (!tok) return cast_uchar "Missing argument";
-	for (i = 0; i < n_languages(); i++)
-		if (!(strcasecmp(cast_const_char language_name(i), cast_const_char tok))) {
+	for (i = -1; i < n_languages(); i++)
+		if (!(casestrcmp(language_name(i), tok))) {
 			set_language(i);
 			mem_free(tok);
 			return NULL;
@@ -1171,13 +1179,17 @@ static unsigned char *term_rd(struct option *o, unsigned char *c)
 	ts->block_cursor = !!((w[0] - '0') & 4);
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
-	if ((i = get_cp_index(w)) == -1) goto err_f;
+	if (!casestrcmp(w, cast_uchar "default")) {
+		i = -1;
+	} else {
+		if ((i = get_cp_index(w)) == -1) goto err_f;
+	}
 #ifndef ENABLE_UTF8
 	if (i == utf8_table) {
 		i = 0;
 	}
 #endif
-	ts->charset = i;
+	ts->character_set = i;
 	mem_free(w);
 	l = get_token_num(&c);
 	if (l < 0) goto ret;
@@ -1228,13 +1240,17 @@ static unsigned char *term2_rd(struct option *o, unsigned char *c)
 	ts->col = w[0] - '0';
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
-	if ((i = get_cp_index(w)) == -1) goto err_f;
+	if (!casestrcmp(w, cast_uchar "default")) {
+		i = -1;
+	} else {
+		if ((i = get_cp_index(w)) == -1) goto err_f;
+	}
 #ifndef ENABLE_UTF8
 	if (i == utf8_table) {
 		i = 0;
 	}
 #endif
-	ts->charset = i;
+	ts->character_set = i;
 	mem_free(w);
 	return NULL;
 	err_f:
@@ -1256,7 +1272,8 @@ static void term_wr(struct option *o, unsigned char **s, int *l)
 		add_to_str(s, l, cast_uchar " ");
 		add_num_to_str(s, l, !!ts->col + !!ts->restrict_852 * 2 + !!ts->block_cursor * 4);
 		add_to_str(s, l, cast_uchar " ");
-		add_to_str(s, l, get_cp_mime_name(ts->charset));
+		if (ts->character_set == -1) add_to_str(s, l, cast_uchar "default");
+		else add_to_str(s, l, get_cp_mime_name(ts->character_set));
 		if (ts->left_margin || ts->right_margin || ts->top_margin || ts->bottom_margin) {
 			add_to_str(s, l, cast_uchar " ");
 			add_num_to_str(s, l, ts->left_margin);
@@ -1275,9 +1292,9 @@ static struct list_head driver_params = { &driver_params, &driver_params };
 struct driver_param *get_driver_param(unsigned char *n)
 {
 	struct driver_param *dp;
-	foreach(dp, driver_params) if (!strcasecmp(cast_const_char dp->name, cast_const_char n)) return dp;
+	foreach(dp, driver_params) if (!casestrcmp(dp->name, n)) return dp;
 	dp = mem_calloc(sizeof(struct driver_param) + strlen(cast_const_char n) + 1);
-	dp->codepage = get_cp_index(cast_uchar "iso-8859-1");
+	dp->kbd_codepage = -1;
 	strcpy(cast_char dp->name, cast_const_char n);
 	dp->shell = mem_calloc(1);
 	dp->nosave = 1;
@@ -1306,19 +1323,21 @@ static unsigned char *dp_rd(struct option *o, unsigned char *c)
 		mem_free(shell);
 		goto err;
 	}
-	if ((cc=get_cp_index(cp)) == -1) {
+	if (!casestrcmp(cp, cast_uchar "default")) {
+		cc = -1;
+	} else if ((cc = get_cp_index(cp)) == -1) {
 		mem_free(n);
 		mem_free(param);
 		mem_free(shell);
 		mem_free(cp);
 		goto err;
 	}
-	dp=get_driver_param(n);
-	dp->codepage=cc;
+	dp = get_driver_param(n);
+	dp->kbd_codepage = cc;
 	if (dp->param) mem_free(dp->param);
-	dp->param=param;
+	dp->param = param;
 	if (dp->shell) mem_free(dp->shell);
-	dp->shell=shell;
+	dp->shell = shell;
 	dp->nosave = 0;
 	mem_free(cp);
 	mem_free(n);
@@ -1331,7 +1350,7 @@ static void dp_wr(struct option *o, unsigned char **s, int *l)
 {
 	struct driver_param *dp;
 	foreachback(dp, driver_params) {
-		if ((!dp->param || !*dp->param) && !dp->codepage && !*dp->shell) continue;
+		if ((!dp->param || !*dp->param) && dp->kbd_codepage == -1 && !*dp->shell) continue;
 		if (dp->nosave) continue;
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, dp->name);
@@ -1340,7 +1359,8 @@ static void dp_wr(struct option *o, unsigned char **s, int *l)
 		add_to_str(s, l, cast_uchar " ");
 		add_quoted_to_str(s, l, dp->shell);
 		add_to_str(s, l, cast_uchar " ");
-		add_to_str(s, l, get_cp_mime_name(dp->codepage));
+		if (dp->kbd_codepage == -1) add_to_str(s, l, cast_uchar "default");
+		else add_to_str(s, l, get_cp_mime_name(dp->kbd_codepage));
 	}
 }
 
@@ -1472,7 +1492,8 @@ static unsigned char *printhelp_cmd(struct option *o, unsigned char ***argv, int
  */
 fprintf(stdout, "%s%s%s%s%s%s\n",
 
-("links [options] URL\n"
+(
+"links [options] URL\n"
 "\n"
 "Options are:\n"
 "\n"
@@ -1547,8 +1568,8 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 " -max-connections <max>\n"
 "  Maximum number of concurrent connections.\n"
 "    (default: 10)\n"
-"\n"),
-(" -max-connections-to-host <max>\n"
+"\n"),(
+" -max-connections-to-host <max>\n"
 "  Maximum number of concurrent connection to a given host.\n"
 "    (default: 2)\n"
 "\n"
@@ -1559,8 +1580,8 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 " -receive-timeout <sec>\n"
 "  Timeout on receive.\n"
 "    (default: 120)\n"
-"\n"),
-(" -unrestartable-receive-timeout <sec>\n"
+"\n"),(
+" -unrestartable-receive-timeout <sec>\n"
 "  Timeout on non restartable connections.\n"
 "    (default: 600)\n"
 "\n"
@@ -1603,8 +1624,8 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 "  Always cache everything regardless of server's caching recomendations.\n"
 "    Many servers deny caching even if their content is not changing\n"
 "    just to get more hits and more money from ads.\n"
-"\n"),
-(" -address-preference <0>/<1>/<2>/<3>/<4>\n"
+"\n"),(
+" -address-preference <0>/<1>/<2>/<3>/<4>\n"
 "    (default 0)\n"
 "  0 - use system default.\n"
 "  1 - prefer IPv4.\n"
@@ -1637,8 +1658,25 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 "    (default 0)\n"
 "  \"1\" causes that Links won't initiate any non-proxy connection.\n"
 "    It is useful for anonymization with tor or similar networks.\n"
-"\n"),
-(" -http-bugs.http10 <0>/<1>\n"
+"\n"
+" -ssl.certificates <0>/<1>/<2>\n"
+"    (default 1)\n"
+"  0 - ignore invalid certificate\n"
+"  1 - warn on invalid certificate\n"
+"  2 - reject invalid certificate\n"
+"\n"
+" -ssl.client-cert-key <filename>\n"
+"  Name of the PEM encoded file with the user private key\n"
+"  for client certificate authentication.\n"
+"\n"
+" -ssl.client-cert-crt <filename>\n"
+"  Name of the PEM encoded file with the user certificate\n"
+"  for client certificate authentication.\n"
+"\n"
+" -ssl.client-cert-password <text>\n"
+"  Password for the user private key.\n"
+"\n"),(
+" -http-bugs.http10 <0>/<1>\n"
 "    (default 0)\n"
 "  \"1\" forces using only HTTP/1.0 protocol. (useful for buggy servers\n"
 "    that claim to be HTTP/1.1 compliant but are not)\n"
@@ -1888,8 +1926,8 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 "\n"
 " -html-g-ignore-document-color <0>/<1>\n"
 "  Ignore colors specified in html document in graphics mode.\n"
-"\n"),
-("Keys:\n"
+"\n"),(
+"Keys:\n"
 "	ESC	  display menu\n"
 "	^C	  quit\n"
 "	^P	  scroll up\n"
@@ -1997,6 +2035,7 @@ int aggressive_cache = 1;
 
 struct ipv6_options ipv6_options = { ADDR_PREFERENCE_DEFAULT };
 struct proxies proxies = { "", "", "", "", "", 0 };
+struct ssl_options ssl_options = { SSL_WARN_ON_INVALID_CERTIFICATE, "", "", "" };
 struct http_options http_options = { 0, 1, 1, 0, 0, 0, 0, { 0, 0, REFERER_REAL_SAME_SERVER, "", "", "" } };
 struct ftp_options ftp_options = { "somebody@host.domain", 0, 0, 0, 1 };
 struct smb_options smb_options = { 0 };
@@ -2112,6 +2151,12 @@ static struct option links_options[] = {
 	{1, gen_cmd, str_rd, NULL, 0, MAX_STR_LEN, proxies.dns_append, "-append_text_to_dns_lookups", NULL}, /* old version incorrectly saved it with '-' */
 	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, proxies.dns_append, "append_text_to_dns_lookups", "append-text-to-dns-lookups"},
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &proxies.only_proxies, "only_proxies", "only-proxies"},
+	{1, gen_cmd, num_rd, num_wr, 0, 2, &ssl_options.certificates, "ssl.certificates", "ssl.certificates"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, &ssl_options.client_cert_key, "ssl.client_cert_key", "ssl.client-cert-key"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, &ssl_options.client_cert_crt, "ssl.client_cert_crt", "ssl.client-cert-crt"},
+	{1, gen_cmd, str_rd, NULL, 0, MAX_STR_LEN, &ssl_options.client_cert_password, NULL, "ssl.client-cert-password"},
+	{1, gen_cmd, str_rd, NULL, 0, MAX_STR_LEN, &ssl_options.client_cert_key, "client_cert_key", "http.client_cert_key"}, /* backward compatibility with Debian patches */
+	{1, gen_cmd, str_rd, NULL, 0, MAX_STR_LEN, &ssl_options.client_cert_crt, "client_cert_crt", "http.client_cert_crt"}, /* backward compatibility with Debian patches */
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &http_options.http10, "http_bugs.http10", "http-bugs.http10"},
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &http_options.allow_blacklist, "http_bugs.allow_blacklist", "http-bugs.allow-blacklist"},
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &http_options.bug_302_redirect, "http_bugs.bug_302_redirect", "http-bugs.bug-302-redirect"},
@@ -2285,8 +2330,7 @@ void save_url_history(void)
 	unsigned char *history_file;
 	unsigned char *hs;
 	int hsl = 0;
-	int i = 0;
-	if (anonymous || !save_history) return;
+	if (anonymous || !save_history || proxies.only_proxies) return;
 
 	/* Must have been called after init_home */
 	if (!links_home) return;
@@ -2296,7 +2340,7 @@ void save_url_history(void)
 	hsl = 0;
 	foreachback(hi, goto_url_history.items) {
 		if (!*hi->d || hi->d[0] == ' ' || strchr(cast_const_char hi->d, 10) || strchr(cast_const_char hi->d, 13)) continue;
-		if (!url_not_saveable(hi->d) && i++ <= MAX_HISTORY_ITEMS) {
+		if (!url_not_saveable(hi->d)) {
 			add_to_str(&hs, &hsl, hi->d);
 			add_to_str(&hs, &hsl, cast_uchar NEWLINE);
 		}
