@@ -20,12 +20,6 @@
  * Leave it commented up for normal operation and releases! */
 /* #define REPORT_UNKNOWN 1 */
 
-double sRGB_gamma=0.45455;      /* For HTML, which runs
-				 * according to sRGB standard. Number
-				 * in HTML tag is linear to photons raised
-				 * to this power.
-				 */
-
 unsigned aspect=65536; /* aspect=65536 for 320x240
 			* aspect=157286 for 640x200 (tall pixels)
 			* Defines aspect ratio of screen pixels.
@@ -48,39 +42,39 @@ unsigned aspect=65536; /* aspect=65536 for 320x240
  * If you change them, don't wonder if the letters start to look strange.
  * The numers in the comments denote which height the line applies for.
  */
-static const float fancy_constants[64]={
+static_const float fancy_constants[64]={
 	(float)0,(float)3,	/*  1 */
-	(float).1,(float)3,   	/*  2 */
-	(float).2,(float)3,   	/*  3 */
-	(float).3,(float)2.9,  	/*  4 */
-	(float).4,(float)2.7,  	/*  5 */
-	(float).4,(float)2.5,  	/*  6 */
-	(float).4,(float)2,   	/*  7 */
-	(float).5,(float)2, 	/*  8 */
-	(float).4,(float)2, 	/*  9 */
-	(float).38,(float)1.9, 	/* 10 */
-	(float).36,(float)1.8, 	/* 11 */
-	(float).33,(float)1.7, 	/* 12 */
-	(float).30,(float)1.6, 	/* 13 */
-	(float).25,(float)1.5, 	/* 14 */
-	(float).20,(float)1.5, 	/* 15 */
-	(float).15,(float)1.5, 	/* 16 */
-	(float).14,(float)1.5, 	/* 17 */
-	(float).13,(float)1.5, 	/* 18 */
-	(float).12,(float)1.5, 	/* 19 */
-	(float).12,(float)1.5, 	/* 20 */
-	(float).12,(float)1.5, 	/* 21 */
-	(float).12,(float)1.5, 	/* 22 */
-	(float).11,(float)1.5, 	/* 23 */
-	(float).10,(float)1.4, 	/* 24 */
-	(float).09,(float)1.3, 	/* 25 */
-	(float).08,(float)1.3, 	/* 26 */
-	(float).04,(float)1.2, 	/* 27 */
-	(float).04,(float)1.2, 	/* 28 */
-	(float).02,(float)1.1, 	/* 29 */
-	(float).02,(float)1.1, 	/* 30 */
-	(float).01,(float)1, 	/* 31 */
-	(float).01,(float)1  	/* 32 */
+	(float).1,(float)3,	/*  2 */
+	(float).2,(float)3,	/*  3 */
+	(float).3,(float)2.9,	/*  4 */
+	(float).4,(float)2.7,	/*  5 */
+	(float).4,(float)2.5,	/*  6 */
+	(float).4,(float)2,	/*  7 */
+	(float).5,(float)2,	/*  8 */
+	(float).4,(float)2,	/*  9 */
+	(float).38,(float)1.9,	/* 10 */
+	(float).36,(float)1.8,	/* 11 */
+	(float).33,(float)1.7,	/* 12 */
+	(float).30,(float)1.6,	/* 13 */
+	(float).25,(float)1.5,	/* 14 */
+	(float).20,(float)1.5,	/* 15 */
+	(float).15,(float)1.5,	/* 16 */
+	(float).14,(float)1.5,	/* 17 */
+	(float).13,(float)1.5,	/* 18 */
+	(float).12,(float)1.5,	/* 19 */
+	(float).12,(float)1.5,	/* 20 */
+	(float).12,(float)1.5,	/* 21 */
+	(float).12,(float)1.5,	/* 22 */
+	(float).11,(float)1.5,	/* 23 */
+	(float).10,(float)1.4,	/* 24 */
+	(float).09,(float)1.3,	/* 25 */
+	(float).08,(float)1.3,	/* 26 */
+	(float).04,(float)1.2,	/* 27 */
+	(float).04,(float)1.2,	/* 28 */
+	(float).02,(float)1.1,	/* 29 */
+	(float).02,(float)1.1,	/* 30 */
+	(float).01,(float)1,	/* 31 */
+	(float).01,(float)1	/* 32 */
 };
 
 
@@ -118,7 +112,7 @@ static struct lru font_cache;
 			 * for letters with an image under them )
 			 */
 
-#if defined(__i686__) || defined(__athlon__) || defined(__SSE2__) || defined(__x86_64__) || defined(__alpha__)
+#if defined(__i686__) || defined(__athlon__) || defined(__SSE2__) || defined(__x86_64__) || defined(__alpha) || defined(__hppa)
 /*
  * On modern processors it is faster to do this in floating point.
  * Do it only if we are sure that the coprocessor is present.
@@ -140,7 +134,7 @@ typedef unsigned long scale_t;
  * black or 255 is black doesn't matter.
  */
 
-static inline void add_col_gray(unsigned *col_buf, unsigned char *ptr, int
+static inline void add_col_gray(unsigned *my_restrict col_buf, unsigned char *my_restrict ptr, int
 		line_skip, int n, unsigned weight)
 {
 	for (;n;n--){
@@ -151,7 +145,7 @@ static inline void add_col_gray(unsigned *col_buf, unsigned char *ptr, int
 }
 
  /* We assume unsigned short holds at least 16 bits. */
-static inline void add_row_gray(unsigned *row_buf, unsigned char *ptr, int n,
+static inline void add_row_gray(unsigned *my_restrict row_buf, unsigned char *my_restrict ptr, int n,
 	unsigned weight)
 {
 	for (;n;n--){
@@ -163,7 +157,7 @@ static inline void add_row_gray(unsigned *row_buf, unsigned char *ptr, int n,
 
 /* line_skip is in pixels. The column contains the whole pixels (R G B)
  * We assume unsigned short holds at least 16 bits. */
-static inline void add_col_color(scale_t *col_buf, unsigned short *ptr,
+static inline void add_col_color(scale_t *my_restrict col_buf, unsigned short *my_restrict ptr,
 	int line_skip, int n, ulonglong weight)
 {
 	if (!weight) return;
@@ -180,7 +174,7 @@ static inline void add_col_color(scale_t *col_buf, unsigned short *ptr,
 	} else
 #endif
 	{
-		scale_t w = (scale_t)weight;
+		scale_t w = (scale_t)(longlong)weight;
 		for (;n;n--){
 			col_buf[0]+=w*ptr[0];
 			col_buf[1]+=w*ptr[1];
@@ -193,7 +187,7 @@ static inline void add_col_color(scale_t *col_buf, unsigned short *ptr,
 
 /* n is in pixels. pixel is 3 unsigned shorts in series */
  /* We assume unsigned short holds at least 16 bits. */
-static inline void add_row_color(scale_t *row_buf, unsigned short *ptr,
+static inline void add_row_color(scale_t *my_restrict row_buf, unsigned short *my_restrict ptr,
 	int n, ulonglong weight)
 {
 	if (!weight) return;
@@ -210,7 +204,7 @@ static inline void add_row_color(scale_t *row_buf, unsigned short *ptr,
 	} else
 #endif
 	{
-		scale_t w = (scale_t)weight;
+		scale_t w = (scale_t)(longlong)weight;
 		for (;n;n--){
 			row_buf[0]+=w*ptr[0];
 			row_buf[1]+=w*ptr[1];
@@ -222,7 +216,7 @@ static inline void add_row_color(scale_t *row_buf, unsigned short *ptr,
 }
 
 /* We assume unsigned holds at least 32 bits */
-static inline void emit_and_bias_col_gray(unsigned *col_buf, unsigned char *out,
+static inline void emit_and_bias_col_gray(unsigned *my_restrict col_buf, unsigned char *my_restrict out,
 	int line_skip, int n, unsigned weight)
 {
 	unsigned half=weight>>1;
@@ -235,7 +229,7 @@ static inline void emit_and_bias_col_gray(unsigned *col_buf, unsigned char *out,
 }
 
 /* We assume unsigned holds at least 32 bits */
-static inline void emit_and_bias_row_gray(unsigned *row_buf, unsigned char *out,
+static inline void emit_and_bias_row_gray(unsigned *my_restrict row_buf, unsigned char *my_restrict out,
 	int n, unsigned weight)
 {
 	unsigned half=weight>>1;
@@ -247,13 +241,13 @@ static inline void emit_and_bias_row_gray(unsigned *row_buf, unsigned char *out,
 }
 
 /* We assume unsigned holds at least 32 bits */
-static inline void bias_buf_gray(unsigned *col_buf, int n, unsigned half)
+static inline void bias_buf_gray(unsigned *my_restrict col_buf, int n, unsigned half)
 {
 	for (;n;n--) *col_buf++=half;
 }
 
 /* We assume unsigned holds at least 32 bits */
-static inline void bias_buf_color(scale_t *col_buf, int n, scale_t half)
+static inline void bias_buf_color(scale_t *my_restrict col_buf, int n, scale_t half)
 {
 	for (;n;n--){
 		col_buf[0]=half;
@@ -273,8 +267,8 @@ static inline void bias_buf_color(scale_t *col_buf, int n, scale_t half)
 /* line skip is in pixels. Pixel is 3*unsigned short */
 /* We assume unsigned holds at least 32 bits */
 /* We assume unsigned short holds at least 16 bits. */
-static inline void emit_and_bias_col_color(scale_t *col_buf,
-	unsigned short *out, int line_skip, int n, unsigned weight)
+static inline void emit_and_bias_col_color(scale_t *my_restrict col_buf,
+	unsigned short *my_restrict out, int line_skip, int n, unsigned weight)
 {
 	scale_t half=(scale_t)weight / 2;
 #ifdef USE_FP_SCALE
@@ -300,8 +294,8 @@ static inline void emit_and_bias_col_color(scale_t *col_buf,
 /* n is in pixels. pixel is 3 unsigned shorts in series. */
 /* We assume unsigned holds at least 32 bits */
 /* We assume unsigned short holds at least 16 bits. */
-static inline void emit_and_bias_row_color(scale_t *row_buf,
-	unsigned short *out, int n, unsigned weight)
+static inline void emit_and_bias_row_color(scale_t *my_restrict row_buf,
+	unsigned short *my_restrict out, int n, unsigned weight)
 {
 	scale_t half=(scale_t)weight / 2;
 #ifdef USE_FP_SCALE
@@ -377,14 +371,14 @@ static inline void enlarge_gray_horizontal(unsigned char *in, int ix, int y,
 	/* ztracena zeme :) */
 }
 
-static inline ulonglong multiply_int(int a, int b)
+static inline longlong multiply_int(int a, int b)
 {
 #ifndef HAVE_LONG_LONG
 	volatile
 #endif
-	ulonglong result = (ulonglong)a * (ulonglong)b;
-#ifndef __TINYC__
-	if (result / a != (ulonglong)b) {
+	longlong result = (ulonglong)a * (ulonglong)b;
+#if !defined(__TINYC__) && !defined(HAVE_LONG_LONG)
+	if (result / a != (longlong)b) {
 		/*fprintf(stderr, "%lld, %lld, %d, %d\n", result / a, result, a, b);*/
 		overflow();
 	}
@@ -733,8 +727,8 @@ static inline void enlarge_color_vertical(unsigned short *in, int x, int iy,
 	}
 	if (iy==1){
 		for (;oy;oy--){
-	       		memcpy(out,in,3*x*sizeof(*out));
-	       		out+=3*x;
+			memcpy(out,in,3*x*sizeof(*out));
+			out+=3*x;
 		}
 		mem_free(in);
 		return;
@@ -1083,7 +1077,7 @@ void scale_color(unsigned short *in, int ix, int iy, unsigned short **out,
 /* Fills a block with given color. length is number of pixels. pixel is a
  * tribyte. 24 bits per pixel.
  */
-void mix_one_color_24(unsigned char *dest, int length,
+void mix_one_color_24(unsigned char *my_restrict dest, int length,
 		   unsigned char r, unsigned char g, unsigned char b)
 {
 	for (;length;length--){
@@ -1098,7 +1092,7 @@ void mix_one_color_24(unsigned char *dest, int length,
  * tribyte. 48 bits per pixel.
  * We assume unsigned short holds at least 16 bits.
  */
-void mix_one_color_48(unsigned short *dest, int length,
+void mix_one_color_48(unsigned short *my_restrict dest, int length,
 		   unsigned short r, unsigned short g, unsigned short b)
 {
 	for (;length;length--){
@@ -1116,7 +1110,7 @@ void mix_one_color_48(unsigned short *dest, int length,
  * alpha is 8-bit, rgb are all 16-bit
  * We assume unsigned short holds at least 16 bits.
  */
-static inline void mix_two_colors(unsigned short *dest, unsigned char *alpha,
+static inline void mix_two_colors(unsigned short *my_restrict dest, unsigned char *my_restrict alpha,
 	int length ,unsigned short r0, unsigned short g0, unsigned short b0,
 	unsigned short r255, unsigned short g255, unsigned short b255)
 {
@@ -1145,8 +1139,8 @@ static inline void mix_two_colors(unsigned short *dest, unsigned char *alpha,
 }
 
 /* We assume unsigned short holds at least 16 bits. */
-void agx_and_uc_32_to_48_table(unsigned short *dest,
-		const unsigned char *src, int lenght, unsigned short *table,
+void agx_and_uc_32_to_48_table(unsigned short *my_restrict dest,
+		const unsigned char *my_restrict src, int lenght, unsigned short *my_restrict table,
 		unsigned short rb, unsigned short gb, unsigned short bb)
 {
 	unsigned char alpha, calpha;
@@ -1185,8 +1179,8 @@ void agx_and_uc_32_to_48_table(unsigned short *dest,
  * in linear monitor output photon space
  */
 /* We assume unsigned short holds at least 16 bits. */
-void agx_and_uc_32_to_48(unsigned short *dest,
-		const unsigned char *src, int lenght, float red_gamma,
+void agx_and_uc_32_to_48(unsigned short *my_restrict dest,
+		const unsigned char *my_restrict src, int lenght, float red_gamma,
 		float green_gamma, float blue_gamma, unsigned short rb,
 		unsigned short gb, unsigned short bb)
 {
@@ -1208,9 +1202,9 @@ void agx_and_uc_32_to_48(unsigned short *dest,
 		r=fd_pow(r,red_gamma);
 		g=fd_pow(g,green_gamma);
 		b=fd_pow(b,blue_gamma);
-		ri=(unsigned)((r*65535)+0.5);
-		gi=(unsigned)((g*65535)+0.5);
-		bi=(unsigned)((b*65535)+0.5);
+		ri=(unsigned)((r*65535)+(float_double)0.5);
+		gi=(unsigned)((g*65535)+(float_double)0.5);
+		bi=(unsigned)((b*65535)+(float_double)0.5);
 		cmd_limit_16(ri);
 		cmd_limit_16(gi);
 		cmd_limit_16(bi);
@@ -1239,8 +1233,8 @@ void agx_and_uc_32_to_48(unsigned short *dest,
  * in linear monitor output photon space. alpha 255 means full image no background.
  */
 /* We assume unsigned short holds at least 16 bits. */
-void agx_and_uc_64_to_48(unsigned short *dest,
-		const unsigned short *src, int lenght, float red_gamma,
+void agx_and_uc_64_to_48(unsigned short *my_restrict dest,
+		const unsigned short *my_restrict src, int lenght, float red_gamma,
 		float green_gamma, float blue_gamma, unsigned short rb,
 		unsigned short gb, unsigned short bb)
 {
@@ -1262,9 +1256,9 @@ void agx_and_uc_64_to_48(unsigned short *dest,
 		r=fd_pow(r,red_gamma);
 		g=fd_pow(g,green_gamma);
 		b=fd_pow(b,blue_gamma);
-		ri=(unsigned short)(r*65535+0.5);
-		gi=(unsigned short)(g*65535+0.5);
-		bi=(unsigned short)(b*65535+0.5);
+		ri=(unsigned short)(r*65535+(float_double)0.5);
+		gi=(unsigned short)(g*65535+(float_double)0.5);
+		bi=(unsigned short)(b*65535+(float_double)0.5);
 		cmd_limit_16(ri);
 		cmd_limit_16(gi);
 		cmd_limit_16(bi);
@@ -1293,8 +1287,8 @@ void agx_and_uc_64_to_48(unsigned short *dest,
  * dest. src and dest may be identical and it will work. rb, gb, bb are 0-65535
  * in linear monitor output photon space. alpha 255 means full image no background.
  * We assume unsigned short holds at least 16 bits. */
-void agx_and_uc_64_to_48_table(unsigned short *dest,
-		const unsigned short *src, int lenght, unsigned short *gamma_table,
+void agx_and_uc_64_to_48_table(unsigned short *my_restrict dest,
+		const unsigned short *my_restrict src, int lenght, unsigned short *my_restrict gamma_table,
 		unsigned short rb, unsigned short gb, unsigned short bb)
 {
 	unsigned short alpha, calpha;
@@ -1331,8 +1325,8 @@ void agx_and_uc_64_to_48_table(unsigned short *dest,
  * number of triplets. output is input powered to the given gamma, passed into
  * dest. src and dest may be identical and it will work.
  * We assume unsigned short holds at least 16 bits. */
-void agx_48_to_48(unsigned short *dest,
-		const unsigned short *src, int lenght, float red_gamma,
+void agx_48_to_48(unsigned short *my_restrict dest,
+		const unsigned short *my_restrict src, int lenght, float red_gamma,
 		float green_gamma, float blue_gamma)
 {
 	float_double a;
@@ -1343,17 +1337,17 @@ void agx_48_to_48(unsigned short *dest,
 		a=*src;
 		a*=inv_65535;
 		a=fd_pow(a,red_gamma);
-		dest[0]=(unsigned short)((a*65535)+0.5);
+		dest[0]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[0]);
 		a=src[1];
 		a*=inv_65535;
 		a=fd_pow(a,green_gamma);
-		dest[1]=(unsigned short)((a*65535)+0.5);
+		dest[1]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[1]);
 		a=src[2];
 		a*=inv_65535;
 		a=fd_pow(a,blue_gamma);
-		dest[2]=(unsigned short)((a*65535)+0.5);
+		dest[2]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[2]);
 	}
 }
@@ -1362,8 +1356,8 @@ void agx_48_to_48(unsigned short *dest,
  * number of triples. output is input powered to the given gamma, passed into
  * dest. src and dest may be identical and it will work.
  * We assume unsigned short holds at least 16 bits. */
-void agx_48_to_48_table(unsigned short *dest,
-		const unsigned short *src, int lenght, unsigned short *table)
+void agx_48_to_48_table(unsigned short *my_restrict dest,
+		const unsigned short *my_restrict src, int lenght, unsigned short *my_restrict table)
 {
 	for (;lenght;lenght--,src+=3,dest+=3)
 	{
@@ -1377,7 +1371,7 @@ void agx_48_to_48_table(unsigned short *dest,
  * number of triples. output is input powered to the given gamma, passed into
  * dest. src and dest may be identical and it will work.
  * We assume unsigned short holds at least 16 bits. */
-void agx_24_to_48(unsigned short *dest, const unsigned char *src, int
+void agx_24_to_48(unsigned short *my_restrict dest, const unsigned char *my_restrict src, int
 			  lenght, float red_gamma, float green_gamma, float
 			  blue_gamma)
 {
@@ -1389,17 +1383,17 @@ void agx_24_to_48(unsigned short *dest, const unsigned char *src, int
 		a=*src;
 		a*=inv_255;
 		a=fd_pow(a,red_gamma);
-		dest[0]=(unsigned short)((a*65535)+0.5);
+		dest[0]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[0]);
 		a=src[1];
 		a*=inv_255;
 		a=fd_pow(a,green_gamma);
-		dest[1]=(unsigned short)((a*65535)+0.5);
+		dest[1]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[1]);
 		a=src[2];
 		a*=inv_255;
 		a=fd_pow(a,blue_gamma);
-		dest[2]=(unsigned short)((a*65535)+0.5);
+		dest[2]=(unsigned short)((a*65535)+(float_double)0.5);
 		cmd_limit_16(dest[2]);
 	}
 }
@@ -1409,9 +1403,9 @@ void agx_24_to_48(unsigned short *dest, const unsigned char *src, int
  * We assume unsigned short holds at least 16 bits. */
 void make_gamma_table(struct cached_image *cimg)
 {
-	float_double rg=(float_double)(user_gamma/cimg->red_gamma);
-	float_double gg=(float_double)(user_gamma/cimg->green_gamma);
-	float_double bg=(float_double)(user_gamma/cimg->blue_gamma);
+	float_double rg=(float_double)((float_double)user_gamma/cimg->red_gamma);
+	float_double gg=(float_double)((float_double)user_gamma/cimg->green_gamma);
+	float_double bg=(float_double)((float_double)user_gamma/cimg->blue_gamma);
 	int a;
 	unsigned short *ptr_16;
 	unsigned short last_val;
@@ -1423,17 +1417,17 @@ void make_gamma_table(struct cached_image *cimg)
 		ptr_16=mem_alloc(768*sizeof(*(cimg->gamma_table)));
 		cimg->gamma_table=ptr_16;
 		for (a=0;a<256;a++,ptr_16++){
-			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,rg)+0.5);
+			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,rg)+(float_double)0.5);
 			cmd_limit_16(last_val);
 			*ptr_16 = last_val;
 		}
 		for (a=0;a<256;a++,ptr_16++){
-			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,gg)+0.5);
+			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,gg)+(float_double)0.5);
 			cmd_limit_16(last_val);
 			*ptr_16 = last_val;
 		}
 		for (a=0;a<256;a++,ptr_16++){
-			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,bg)+0.5);
+			last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_255,bg)+(float_double)0.5);
 			cmd_limit_16(last_val);
 			*ptr_16 = last_val;
 		}
@@ -1449,21 +1443,21 @@ void make_gamma_table(struct cached_image *cimg)
 		cimg->gamma_table=ptr_16;
 		for (a=0;a<0x10000;a++,ptr_16++){
 			if (!x_slow_fpu || !(a & 0xff)) {
-				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,rg)+0.5);
+				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,rg)+(float_double)0.5);
 				cmd_limit_16(last_val);
 			}
 			*ptr_16 = last_val;
 		}
 		for (a=0;a<0x10000;a++,ptr_16++){
 			if (!x_slow_fpu || !(a & 0xff)) {
-				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,gg)+0.5);
+				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,gg)+(float_double)0.5);
 				cmd_limit_16(last_val);
 			}
 			*ptr_16 = last_val;
 		}
 		for (a=0;a<0x10000;a++,ptr_16++){
 			if (!x_slow_fpu || !(a & 0xff)) {
-				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,bg)+0.5);
+				last_val = (unsigned short)(65535*fd_pow((float_double)a*inv_65535,bg)+(float_double)0.5);
 				cmd_limit_16(last_val);
 			}
 			*ptr_16 = last_val;
@@ -1472,8 +1466,8 @@ void make_gamma_table(struct cached_image *cimg)
 }
 
 /* We assume unsigned short holds at least 16 bits. */
-void agx_24_to_48_table(unsigned short *dest, const unsigned char *src, int
-			  lenght, unsigned short *table)
+void agx_24_to_48_table(unsigned short *my_restrict dest, const unsigned char *my_restrict src, int
+			  lenght, unsigned short *my_restrict table)
 {
 	for (;lenght;lenght--,src+=3,dest+=3)
 	{
@@ -1488,7 +1482,7 @@ void agx_24_to_48_table(unsigned short *dest, const unsigned char *src, int
 unsigned char ags_8_to_8(unsigned char input, float gamma)
 {
 	const float_double inv_255=1/255.;
-	return 255*fd_pow((float_double)input*inv_255,gamma)+0.5;
+	return 255*fd_pow((float_double)input*inv_255,gamma)+(float_double)0.5;
 }
 #endif
 
@@ -1503,7 +1497,7 @@ unsigned short ags_8_to_16(unsigned char input, float gamma)
 	a*=inv_255;
 	a=fd_pow(a,gamma);
 	a*=65535;
-	retval = (unsigned short)(a+0.5);
+	retval = (unsigned short)(a+(float_double)0.5);
 	cmd_limit_16(retval);
 	return retval;
 }
@@ -1513,7 +1507,7 @@ unsigned short ags_8_to_16(unsigned char input, float gamma)
 unsigned char ags_16_to_8(unsigned short input, float gamma)
 {
 	const float_double inv_65535=(float_double)(1/65535.);
-	return (unsigned char)(fd_pow((float_double)input*inv_65535,gamma)*255+0.5);
+	return (unsigned char)(fd_pow((float_double)input*inv_65535,gamma)*255+(float_double)0.5);
 }
 
 /* Input is 0-65535 (16-bit). Output is 0-255 (8-bit)*/
@@ -1522,7 +1516,7 @@ unsigned short ags_16_to_16(unsigned short input, float gamma)
 	unsigned short retval;
 	const float_double inv_65535=(float_double)(1/65535.);
 
-	retval = (unsigned short)(65535*fd_pow((float_double)input*inv_65535,gamma)+0.5);
+	retval = (unsigned short)(65535*fd_pow((float_double)input*inv_65535,gamma)+(float_double)0.5);
 	cmd_limit_16(retval);
 	return retval;
 }
@@ -1771,12 +1765,12 @@ const unsigned char *png_data, int png_length, struct style *style)
 
 	/* Compute the matrix constants from contrast and sharpness */
 	conv0=(1+sharpness)*contrast;
-	conv1=(float)(-sharpness*0.25*contrast);
+	conv1=(float)(-sharpness*(float)0.25*contrast);
 
 	for (y0=y;y0;y0--){
 		for (x0=*x;x0;x0--){
 			/* Convolution */
-			c=(int)(((*i2ptr)*conv0)+i2ptr[-ix]*conv1+i2ptr[-1]*conv1+i2ptr[1]*conv1+i2ptr[ix]*conv1+0.5);
+			c=(int)(((*i2ptr)*conv0)+i2ptr[-ix]*conv1+i2ptr[-1]*conv1+i2ptr[1]*conv1+i2ptr[ix]*conv1+(float)0.5);
 			if (((unsigned)c)>=256) c=c<0?0:255;
 			*dptr=(unsigned char)c;
 			dptr++;
@@ -1787,41 +1781,25 @@ const unsigned char *png_data, int png_length, struct style *style)
 	mem_free(interm2);
 }
 
-static struct font_cache_entry *locked_bw_entry = NULL;
 static struct font_cache_entry *locked_color_entry = NULL;
 
 /* Adds required entry into font_cache and returns pointer to the entry.
- * We assume the entry is FC_COLOR.
  */
 static struct font_cache_entry *supply_color_cache_entry(struct style *style, struct letter *letter)
 {
 	struct font_cache_entry *found, *neww;
 	unsigned short *primary_data;
-	int do_free=0;
-	struct font_cache_entry templat;
 	unsigned short red, green, blue;
 	unsigned bytes_consumed;
 
-	templat.bitmap.y=style->height;
-	templat.type=FC_BW;
-	/* The BW entries will be added only for background images which
-	 * are not yet implemented, but will be in the future.
-	 */
+	found=mem_alloc(sizeof(*found));
+	found->bitmap.y=style->height;
+	load_scaled_char(&(found->bitmap.data),&(found->bitmap.x),
+		found->bitmap.y, letter->begin,
+		letter->length, style);
 
-	found=lru_lookup(&font_cache, &templat, letter->bw_list);
-	if (!found){
-		found=mem_alloc(sizeof(*found));
-		found->bitmap.y=style->height;
-		load_scaled_char(&(found->bitmap.data),&(found->bitmap.x),
-			found->bitmap.y, letter->begin,
-			letter->length, style);
-		do_free=1;
-	} else {
-		locked_bw_entry = found;
-	}
 	neww=mem_alloc(sizeof(*neww));
 	locked_color_entry = neww;
-	neww->type=FC_COLOR;
 	neww->bitmap=found->bitmap;
 	neww->r0=style->r0;
 	neww->g0=style->g0;
@@ -1843,9 +1821,9 @@ static struct font_cache_entry *supply_color_cache_entry(struct style *style, st
 	mix_two_colors(primary_data, found->bitmap.data,
 		found->bitmap.x*found->bitmap.y,
 		red,green,blue,
-		ags_8_to_16(style->r1,(float)(user_gamma/sRGB_gamma)),
-		ags_8_to_16(style->g1,(float)(user_gamma/sRGB_gamma)),
-		ags_8_to_16(style->b1,(float)(user_gamma/sRGB_gamma))
+		ags_8_to_16(style->r1,(float)((float)user_gamma/(float)sRGB_gamma)),
+		ags_8_to_16(style->g1,(float)((float)user_gamma/(float)sRGB_gamma)),
+		ags_8_to_16(style->b1,(float)((float)user_gamma/(float)sRGB_gamma))
 	);
 	if (display_optimize){
 		/* A correction for LCD */
@@ -1862,12 +1840,10 @@ static struct font_cache_entry *supply_color_cache_entry(struct style *style, st
 	skip_dither:
 	mem_free(primary_data);
 	drv->register_bitmap(&(neww->bitmap));
-	if (do_free){
-		mem_free(found->bitmap.data);
-		mem_free(found);
-	} else {
-		locked_bw_entry = NULL;
-	}
+
+	mem_free(found->bitmap.data);
+	mem_free(found);
+
 	bytes_consumed=neww->bitmap.x*neww->bitmap.y*(drv->depth&7);
 	/* Number of bytes per pixel in passed bitmaps */
 	bytes_consumed+=(int)sizeof(*neww);
@@ -1882,12 +1858,8 @@ static int destroy_font_cache_bottom(void)
 	struct font_cache_entry *bottom;
 	bottom=lru_get_bottom(&font_cache);
 	if (!bottom) return 0;
-	if (bottom == locked_bw_entry || bottom == locked_color_entry) return 0;
-	if (bottom->type==FC_COLOR){
-		drv->unregister_bitmap(&(bottom->bitmap));
-	}else{
-		mem_free(bottom->bitmap.data);
-	}
+	if (bottom == locked_color_entry) return 0;
+	drv->unregister_bitmap(&(bottom->bitmap));
 	mem_free(bottom);
 	lru_destroy_bottom(&font_cache);
 	return 1;
@@ -1923,7 +1895,6 @@ static inline int print_letter(struct graphics_device *device, int x, int y, str
 #ifdef DEBUG
 	if (!letter) internal("print_letter could not find a letter - even not the blotch!");
 #endif /* #ifdef DEBUG */
-	templat.type=FC_COLOR;
 	templat.r0=style->r0;
 	templat.r1=style->r1;
 	templat.g0=style->g0;
@@ -1934,7 +1905,7 @@ static inline int print_letter(struct graphics_device *device, int x, int y, str
 	templat.mono_space=style->mono_space;
 	templat.mono_height=style->mono_height;
 
-	found=lru_lookup(&font_cache, &templat, letter->color_list);
+	found=lru_lookup(&font_cache, &templat, &letter->color_list);
 	if (!found) found=supply_color_cache_entry(style, letter);
 	else locked_color_entry = found;
 	drv->draw_bitmap(device, &(found->bitmap), x, y);
@@ -2057,20 +2028,18 @@ static int compare_font_entries(void *entry, void *templat)
 	struct font_cache_entry*e1=entry;
 	struct font_cache_entry*e2=templat;
 
-	if (e1->type==FC_COLOR){
-		return(
-		 (e1->r0!=e2->r0)||
-		 (e1->g0!=e2->g0)||
-		 (e1->b0!=e2->b0)||
-		 (e1->r1!=e2->r1)||
-		 (e1->g1!=e2->g1)||
-		 (e1->b1!=e2->b1)||
-		 (e1->bitmap.y!=e2->bitmap.y)||
-		 (e1->mono_space!=e2->mono_space)||
-		 (e1->mono_space>=0&&e1->mono_height!=e2->mono_height));
-	}else{
-		return e1->bitmap.y!=e2->bitmap.y;
+	if (e1->r0 != e2->r0) return 1;
+	if (e1->g0 != e2->g0) return 1;
+	if (e1->b0 != e2->b0) return 1;
+	if (e1->r1 != e2->r1) return 1;
+	if (e1->g1 != e2->g1) return 1;
+	if (e1->b1 != e2->b1) return 1;
+	if (e1->bitmap.y != e2->bitmap.y) return 1;
+	if (e1->mono_space != e2->mono_space) return 1;
+	if (e1->mono_space >= 0) {
+		if (e1->mono_height != e2->mono_height) return 1;
 	}
+	return 0;
 }
 
 /* If the cache already exists, it is destroyed and reallocated. If you call it with the same
@@ -2407,9 +2376,9 @@ long real_dip_get_color_sRGB(int rgb)
 	rgb = hack_rgb(rgb);
 
 	round_color_sRGB_to_48(&r,&g,&b,rgb);
-	r=ags_16_to_8(r,(float)(1/display_red_gamma));
-	g=ags_16_to_8(g,(float)(1/display_green_gamma));
-	b=ags_16_to_8(b,(float)(1/display_blue_gamma));
+	r=ags_16_to_8(r,(float)(1/(float)display_red_gamma));
+	g=ags_16_to_8(g,(float)(1/(float)display_green_gamma));
+	b=ags_16_to_8(b,(float)(1/(float)display_blue_gamma));
 	new_rgb=b|(g<<8)|(r<<16);
 	gamma_cache_rgb = rgb;
 	/* The get_color takes values with gamma of display_*_gamma */
@@ -2424,7 +2393,7 @@ void get_links_icon(unsigned char **data, int *width, int *height, int *skip, in
 {
 	struct bitmap b;
 	unsigned short *tmp1;
-	float g=(float)(user_gamma/sRGB_gamma);
+	float g=(float)((float)user_gamma/(float)sRGB_gamma);
 
 	b.x=48;
 	b.y=48;
