@@ -5,6 +5,8 @@
 #include <pc.h>
 #include <dpmi.h>
 #include <bios.h>
+#include <go32.h>
+#include <libc/dosio.h>
 #include <sys/exceptn.h>
 #include <sys/movedata.h>
 
@@ -77,7 +79,7 @@ static int dos_mouse_last_y;
 static int dos_mouse_last_button;
 static ttime dos_mouse_time;
 
-static struct event *dos_mouse_queue = DUMMY;
+static struct links_event *dos_mouse_queue = DUMMY;
 static int dos_mouse_queue_n;
 
 static void (*txt_mouse_handler)(void *, unsigned char *, int) = NULL;
@@ -150,7 +152,7 @@ static void dos_mouse_enqueue(int x, int y, int b)
 		dos_mouse_queue_n--;
 		goto set_last;
 	}
-	dos_mouse_queue = mem_realloc(dos_mouse_queue, (dos_mouse_queue_n + 1) * sizeof(struct event));
+	dos_mouse_queue = mem_realloc(dos_mouse_queue, (dos_mouse_queue_n + 1) * sizeof(struct links_event));
 set_last:
 	dos_mouse_queue[dos_mouse_queue_n].ev = EV_MOUSE;
 	dos_mouse_queue[dos_mouse_queue_n].x = x;
@@ -266,11 +268,11 @@ static int dos_mouse_event(void)
 {
 	if (dos_mouse_queue_n) {
 		if (!F && txt_mouse_handler) {
-			struct event *q = dos_mouse_queue;
+			struct links_event *q = dos_mouse_queue;
 			int ql = dos_mouse_queue_n;
 			dos_mouse_queue = DUMMY;
 			dos_mouse_queue_n = 0;
-			txt_mouse_handler(txt_mouse_data, (unsigned char *)(void *)q, ql * sizeof(struct event));
+			txt_mouse_handler(txt_mouse_data, (unsigned char *)(void *)q, ql * sizeof(struct links_event));
 			mem_free(q);
 			return 1;
 		}
@@ -754,7 +756,7 @@ void setcooked(int ctl)
 
 #define RANDOM_POOL_SIZE	65536
 
-void os_seed_random(unsigned char **pool, unsigned *pool_size)
+void os_seed_random(unsigned char **pool, int *pool_size)
 {
 	unsigned *random_pool, *tmp_pool;
 	int a, i;
@@ -778,9 +780,10 @@ void init_os(void)
 	init_seq_len = strlen(cast_const_char init_seq);
 
 	/* preload the packet driver */
-	EINTRLOOP(s, socket(PF_INET, SOCK_STREAM, IPPROTO_TCP));
-	if (s >= 0)
-		close(s);
+	s = c_socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+	if (s >= 0) {
+		EINTRLOOP(rs, close(s));
+	}
 
 	tcp_cbreak(1);
 
@@ -797,6 +800,43 @@ void terminate_osdep(void)
 		mem_free(screen_backbuffer);
 }
 
+int os_default_language(void)
+{
+	__dpmi_regs r;
+	memset(&r, 0, sizeof r);
+	r.x.ax = 0x3800;
+	r.x.dx = __tb_offset;
+	r.x.ds = __tb_segment;
+	__dpmi_int(0x21, &r);
+	if (!(r.x.flags & 1)) {
+		return get_country_language(r.x.bx);
+	}
+	return -1;
+}
+
+int os_default_charset(void)
+{
+	__dpmi_regs r;
+	memset(&r, 0, sizeof r);
+	r.x.ax = 0x6601;
+	__dpmi_int(0x21, &r);
+	if (!(r.x.flags & 1)) {
+		unsigned char a[8];
+		int cp;
+		snprintf(cast_char a, sizeof a, "%d", r.x.bx);
+		if ((cp = get_cp_index(a)) >= 0 && cp != utf8_table)
+			return cp;
+	}
+	return 0;
+}
+
+int dos_is_bw(void)
+{
+	unsigned char cfg;
+	dosmemget(0x410, sizeof cfg, &cfg);
+	return (cfg & 0x30) == 0x30;
+}
+
 #else
 
 typedef int dos_c_no_empty_unit;
