@@ -10,11 +10,40 @@
 
 #ifdef G
 
+#include "links.h"
+
 #ifdef HAVE_MATH_H
 #include <math.h>
 #endif
 
-#include "links.h"
+struct gif_decoder{
+	unsigned char *color_map; /* NULL if no color map, otherwise a block of 768 bytes, red, green, blue,
+				     in sRGB, describing color slots 0...255. */
+	int state; /* State of the automatus finitus recognizing the GIF
+		    * format.  0 is initial. */
+	/* Image width, height, bits per pixel, bytes per line, number of bit planes */
+	int im_width;
+	int im_height;
+	int im_bpp; /* Bits per pixel (in codestream) */
+	int code_size;
+	int initial_code_size;
+	int remains; /* Used to skip unwanted blocks in raster data */
+	struct gif_table_entry table[4096]; /* NULL when not present */
+	unsigned char *actual_line; /* Points to actual line in goi->buffer */
+	unsigned char tbuf[16]; /* For remembering headers and similar things. */
+	int tlen; /* 0 in the beginning . tbuf length */
+	int xoff, yoff;
+	int interl_dist;
+	int bits_read; /* How many bits are already read from the symbol
+			* Currently being read */
+	int last_code; /* This is somehow used in the decompression algorithm */
+	int read_code;
+	int CC;
+	int EOI;
+	int table_pos;
+	int first_code;
+	int transparent;
+};
 
 /****************************** Functions *************************************/
 
@@ -38,7 +67,7 @@ static void alloc_color_map(int colors)
  deco->color_map=mem_alloc(colors*3*sizeof(*(deco->color_map)));
 }
 
-/* 
+/*
    Initialize code table: construct codes 0...(1<<code_size)-1 with values
    0...(1<<code_size)-1 Codes (1<<code_size) and (1<<code_size)+1 are
    left intact -- they are of no use.
@@ -69,17 +98,17 @@ static void init_table(void)
  deco->table_pos=i+2;
  for (;i<4096;i++)
  {
-  deco->table[i].pointer=-2; 
+  deco->table[i].pointer=-2;
  }
  deco->code_size++;
  deco->last_code=0;
 }
 
-/* 
+/*
    Outputs a single pixel.
    if end_callback_hit gets set, do not send any more data in.
 */
-static inline void 
+static inline void
 output_pixel(int c)
 {
 	struct gif_decoder *deco;
@@ -99,21 +128,21 @@ output_pixel(int c)
 		deco->xoff=0;
 		global_cimg->rows_added=1;
 		if (deco->interl_dist==1)
-		{ 
+		{
 			if (global_cimg->strip_optimized){
 				buffer_to_bitmap_incremental(cimg
 					,deco->actual_line, 1, deco->yoff,
 					cimg->dregs, 1);
-	  		}else{
-	  			deco->actual_line+=deco->im_width*3;
-	  		}
+			}else{
+				deco->actual_line+=deco->im_width*3;
+			}
 			deco->yoff++;
-  		}else{
+		}else{
 			int skip=deco->interl_dist&15;
-   			int n=(deco->interl_dist==24)
+			int n=(deco->interl_dist==24)
 				?8:(deco->interl_dist>>1);
 			unsigned char *ptr;
-   			int y;
+			int y;
 
 			ptr=deco->actual_line+deco->im_width*3;
 			for (y=deco->yoff+1;y<deco->im_height
@@ -135,12 +164,12 @@ output_pixel(int c)
 			}else{
 				deco->interl_dist=(deco->interl_dist==24)
 					?8:(deco->interl_dist>>1);
-	    			deco->yoff=deco->interl_dist>>1;
+				deco->yoff=deco->interl_dist>>1;
 				deco->actual_line=global_cimg
 					->buffer+deco->yoff*3*deco->im_width;
-   			}
-  		}
- 	}
+			}
+		}
+	}
 }
 
 /* Finds the first char of output string for given codeword. */
@@ -162,7 +191,7 @@ find_first(int c)
 }
 
 /* GIF code
-   Supply a code and it outputs the string for c. 
+   Supply a code and it outputs the string for c.
    if end_callback_hit is set then it should not be called anymore.
 */
 static inline void
@@ -170,13 +199,13 @@ output_string(int c)
 {
 	int pos=0;
 	struct gif_decoder *deco;
- 
+
 	deco=global_cimg->decoder;
 	while(1){
 		if (pos==4096){
-	  		/* Cycle in string */
-		  	end_callback_hit=1;
-		  	return;
+			/* Cycle in string */
+			end_callback_hit=1;
+			return;
 		}
 		deco->table[pos].garbage=deco->table[c].end_char;
 		pos++;
@@ -200,9 +229,10 @@ static inline void
 add_table(struct gif_decoder *deco,int end_char,short pointer)
 {
 	if (deco->table_pos>=4096){
-	 	end_callback_hit=1;
-	 	return; /* Overflow */
- 	}
+		/* ignore overflows because they may happen */
+		/*end_callback_hit=1;*/
+		return; /* Overflow */
+	}
 	deco->table[deco->table_pos].end_char=(unsigned char)end_char;
 	deco->table[deco->table_pos].pointer=pointer;
 	deco->table_pos++;
@@ -222,19 +252,19 @@ static inline void
 accept_code(struct gif_decoder *deco,int c)
 {
  int k;
- 
+
  if (c>4096||c<0) return; /* Erroneous code word will be ignored */
  if (c==deco->CC) {
   init_table();
   return;
  }
- 
+
  if (c==deco->EOI)
  {
   end_callback_hit=1;
   return;
  }
- 
+
  if (deco->first_code)
  {
   deco->first_code=0;
@@ -245,7 +275,7 @@ accept_code(struct gif_decoder *deco,int c)
   deco->last_code=c;
   return;
  }
- 
+
  if (c>=deco->table_pos)
  {
   /* The code is not in the table */
@@ -256,10 +286,10 @@ accept_code(struct gif_decoder *deco,int c)
   /* The code is in code table */
   k=find_first(c);
  }
-  
+
  add_table(deco,k,deco->last_code);
  if (end_callback_hit) return;
- 
+
  /* Output the string for code */
  output_string(c);
  if (end_callback_hit) return;
@@ -276,7 +306,7 @@ accept_byte(unsigned char c)
 {
  int original_code_size;
  struct gif_decoder *deco;
- 
+
  deco=global_cimg->decoder;
  deco->read_code|=(int)((unsigned long)c<<deco->bits_read);
  deco->bits_read+=8;
@@ -299,7 +329,7 @@ static void implant_transparent(struct gif_decoder *deco)
 	if (deco->transparent>=0&&deco->transparent<(1<<deco->im_bpp)){
 		if (global_cimg->strip_optimized){
 			compute_background_8(deco->color_map+3*deco->transparent,
-				global_cimg);	
+				global_cimg);
 		}else{
 			memcpy(deco->color_map+3*deco->transparent
 				,deco->actual_line,3);
@@ -323,12 +353,12 @@ static int gif_dimensions_known(void)
 	/* Run strip_optimized only from 65536 pixels up */
 	return header_dimensions_known(global_cimg);
 }
-	
+
 /* Accepts one byte from GIF codestream
    Caller is responsible for destorying the decoder when
    end_callback_hit is 1.
 */
-static inline void 
+static inline void
 gif_accept_byte(unsigned char c)
 {
 	struct gif_decoder *deco;
@@ -343,9 +373,8 @@ gif_accept_byte(unsigned char c)
 		if (deco->tlen>=13){
 			if (strncmp(cast_const_char deco->tbuf,"GIF87a",6)
 				&&strncmp(cast_const_char deco->tbuf,"GIF89a",6)){
-				bad_file:
-	   			end_callback_hit=1;
-	   			return; /* Invalid GIF header */
+				end_callback_hit=1;
+				return; /* Invalid GIF header */
 			}
 			deco->im_bpp=(deco->tbuf[10]&7)+1;
 			deco->tlen=0;
@@ -357,10 +386,9 @@ gif_accept_byte(unsigned char c)
 				alloc_color_map(1<<deco->im_bpp);
 				deco->state=1;
 			}
-			if (deco->tbuf[10] & 8 || deco->tbuf[12]) {
-				/* Test for corrupted file */
-	   			goto bad_file;
-			}
+			/*if (deco->tbuf[10] & 8 || deco->tbuf[12]) {
+				goto bad_file;
+			}*/
 		}
 		break;
 
@@ -372,7 +400,7 @@ gif_accept_byte(unsigned char c)
 			deco->tlen=0;
 		}
 		break;
-  
+
 		case 2: /* Reading garbage before and one ',' or '!' in GIF */
 		switch (c){
 			case ',':
@@ -384,7 +412,7 @@ gif_accept_byte(unsigned char c)
 			deco->state=3;
 			deco->tlen=0;
 			break;
-   
+
 			case '!':
 			deco->state=7;
 			break;
@@ -403,7 +431,7 @@ gif_accept_byte(unsigned char c)
 			}
 			if (deco->tbuf[8]&64){
 				/* Interlaced order */
-				deco->interl_dist=24; /* Actually 8, the 16 indicates 
+				deco->interl_dist=24; /* Actually 8, the 16 indicates
 						       * it is the first pass. */
 			}else
 				deco->interl_dist=1;
@@ -428,7 +456,7 @@ gif_accept_byte(unsigned char c)
 				deco->xoff=0;
 				deco->yoff=0;
 			}
-	 	}	 
+		}
 		break;
 
 		case 4: /* Reading local colormap in GIF */
@@ -443,15 +471,15 @@ gif_accept_byte(unsigned char c)
 
 		case 5: /* Reading code size block in GIF */
 		deco->initial_code_size=c;
-  		if (deco->initial_code_size<=1||deco->initial_code_size>8){
+		if (deco->initial_code_size<=1||deco->initial_code_size>8){
 			end_callback_hit=1;
 			return; /* Invalid initial code size */
 		}
 		if (!deco->color_map){
-	 		end_callback_hit=1;
-	 		return;
-  		}
-  		deco->bits_read=0;
+			end_callback_hit=1;
+			return;
+		}
+		deco->bits_read=0;
 		deco->read_code=0;  /* Nothing read */
 		init_table(); /* Decoding is about to begin sets up code_size. */
 		deco->state=6;
@@ -461,7 +489,7 @@ gif_accept_byte(unsigned char c)
 		break;
 
 		case 6: /* Reading image data in GIF */
-	  	if (!deco->remains){
+		if (!deco->remains){
 			/* This byte is a count byte. Feed it into remains. */
 			deco->remains=c;
 			if (!c){
@@ -469,7 +497,7 @@ gif_accept_byte(unsigned char c)
     end_callback_hit=1; /* Don't send any following data */
     return;
    }
-  } 
+  }
   else
   {
    accept_byte(c);
@@ -477,7 +505,7 @@ gif_accept_byte(unsigned char c)
    deco->remains--;
   }
   break;
-  
+
   case 7: /* Reading a byte after '!' in GIF */
   if (c==0xf9) deco->state=9;
   else deco->state=8;
