@@ -205,7 +205,7 @@ static void add_url_to_str(unsigned char **str, int *l, unsigned char *url)
 static void http_send_header(struct connection *c)
 {
 	struct http_connection_info *info;
-	int http10 = http_options.http10 && !SCRUB_HEADERS;
+	int http10 = http_options.http10;
 	int proxy;
 	unsigned char *hdr;
 	unsigned char *h, *u;
@@ -354,7 +354,7 @@ static void add_user_agent(unsigned char **hdr, int *l)
 {
 	add_to_str(hdr, l, cast_uchar "User-Agent: ");
 	if (SCRUB_HEADERS) {
-		add_to_str(hdr, l, cast_uchar "Mozilla/5.0 (Windows NT 6.1; rv:38.0) Gecko/20100101 Firefox/38.0\r\n");
+		add_to_str(hdr, l, cast_uchar "Mozilla/5.0 (Windows NT 6.1; rv:45.0) Gecko/20100101 Firefox/45.0\r\n");
 	} else if (!(*http_options.header.fake_useragent)) {
 		add_to_str(hdr, l, cast_uchar("Links (" VERSION_STRING "; "));
 		add_to_str(hdr, l, system_name);
@@ -384,7 +384,7 @@ static void add_user_agent(unsigned char **hdr, int *l)
 
 static void add_referer(unsigned char **hdr, int *l, unsigned char *url, unsigned char *prev_url)
 {
-	if (SCRUB_HEADERS)
+	if (proxies.only_proxies)
 		return;
 	switch (http_options.header.referer)
 	{
@@ -406,12 +406,12 @@ static void add_referer(unsigned char **hdr, int *l, unsigned char *url, unsigne
 			int brk = 1;
 			if ((h = get_host_name(url))) {
 				if ((j = get_host_name(prev_url))) {
-					if (!strcasecmp(cast_const_char h, cast_const_char j)) brk = 0;
-					else if (!strcasecmp(cast_const_char h, "imageproxy.jxs.cz")) {
+					if (!casestrcmp(h, j)) brk = 0;
+					else if (!casestrcmp(h, cast_uchar "imageproxy.jxs.cz")) {
 						int l = (int)strlen(cast_const_char j);
 						int q = (int)strlen(".blog.cz");
-						if (l > q && !strcasecmp(cast_const_char (j + l - q), ".blog.cz")) brk = 0;
-						else if (!strcasecmp(cast_const_char j, "blog.cz")) brk = 0;
+						if (l > q && !casestrcmp((j + l - q), cast_uchar ".blog.cz")) brk = 0;
+						else if (!casestrcmp(j, cast_uchar "blog.cz")) brk = 0;
 					}
 					mem_free(j);
 				}
@@ -458,7 +458,7 @@ static void add_accept_language(unsigned char **hdr, int *l, struct http_connect
 		} else {
 			int la;
 			la = *l;
-			add_to_str(hdr, l, _(TEXT_(T__ACCEPT_LANGUAGE), NULL));
+			add_to_str(hdr, l, get_text_translation(TEXT_(T__ACCEPT_LANGUAGE), NULL));
 			add_to_str(hdr, l, cast_uchar ",");
 			if (!strstr(cast_const_char(*hdr + la), "en,") &&
 			    !strstr(cast_const_char(*hdr + la), "en;")) add_to_str(hdr, l, cast_uchar "en;q=0.2,");
@@ -842,7 +842,6 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	off_t cf;
 	int state = c->state != S_PROC ? S_GETH : S_PROC;
 	unsigned char *head;
-	unsigned char *cookie, *ch;
 	int a, h = 0, version = 0;	/* against warning */
 	unsigned char *d;
 	struct cache_entry *e;
@@ -897,12 +896,6 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 		retry_connection(c);
 		return;
 	}
-	ch = head;
-	while ((cookie = parse_http_header(ch, cast_uchar "Set-Cookie", &ch))) {
-		unsigned char *host = remove_proxy_prefix(c->url);
-		set_cookie(NULL, host, cookie);
-		mem_free(cookie);
-	}
 	if (h == 100) {
 		mem_free(head);
 		state = S_PROC;
@@ -930,6 +923,15 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 		return;
 	}
 #endif
+	if (h != 401 && h != 407) {
+		unsigned char *cookie;
+		unsigned char *ch = head;
+		while ((cookie = parse_http_header(ch, cast_uchar "Set-Cookie", &ch))) {
+			unsigned char *host = remove_proxy_prefix(c->url);
+			set_cookie(NULL, host, cookie);
+			mem_free(cookie);
+		}
+	}
 	if (h == 204) {
 		mem_free(head);
 		http_end_request(c, 0, 0, S_HTTP_204);
@@ -1003,14 +1005,8 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	}
 #ifdef HAVE_SSL
 	if (c->ssl) {
-		int l = 0;
 		if (e->ssl_info) mem_free(e->ssl_info);
-		e->ssl_info = init_str();
-		add_num_to_str(&e->ssl_info, &l, SSL_get_cipher_bits(c->ssl, NULL));
-		add_to_str(&e->ssl_info, &l, cast_uchar "-bit ");
-		add_to_str(&e->ssl_info, &l, cast_uchar SSL_get_cipher_version(c->ssl));
-		add_to_str(&e->ssl_info, &l, cast_uchar " ");
-		add_to_str(&e->ssl_info, &l, cast_uchar SSL_get_cipher_name(c->ssl));
+		e->ssl_info = get_cipher_string(c->ssl);
 	}
 #endif
 	if (e->redirect) mem_free(e->redirect), e->redirect = NULL;
@@ -1045,7 +1041,7 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	info->length = -1;
 	info->version = version;
 	if ((d = parse_http_header(e->head, cast_uchar "Connection", NULL)) || (d = parse_http_header(e->head, cast_uchar "Proxy-Connection", NULL))) {
-		if (!strcasecmp(cast_const_char d, "close")) info->close = 1;
+		if (!casestrcmp(d, cast_uchar "close")) info->close = 1;
 		mem_free(d);
 	} else if (version < 11) info->close = 1;
 	cf = c->from;
@@ -1053,7 +1049,7 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	if ((d = parse_http_header(e->head, cast_uchar "Content-Range", NULL))) {
 		if (strlen(cast_const_char d) > 6) {
 			d[5] = 0;
-			if (!(strcasecmp(cast_const_char d, "bytes")) && d[6] >= '0' && d[6] <= '9') {
+			if (!(casestrcmp(d, cast_uchar "bytes")) && d[6] >= '0' && d[6] <= '9') {
 				my_strtoll_t f = my_strtoll(d + 6, NULL);
 				if (f >= 0 && (off_t)f >= 0 && (off_t)f == f) c->from = f;
 			}
@@ -1079,14 +1075,14 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 		mem_free(d);
 	}
 	if ((d = parse_http_header(e->head, cast_uchar "Accept-Ranges", NULL))) {
-		if (!strcasecmp(cast_const_char d, "none") && !c->unrestartable) c->unrestartable = 1;
+		if (!casestrcmp(d, cast_uchar "none") && !c->unrestartable) c->unrestartable = 1;
 		mem_free(d);
 	} else {
 		if (!c->unrestartable && !c->from) c->unrestartable = 1;
 	}
 	if (info->bl_flags & BL_NO_RANGE && !c->unrestartable) c->unrestartable = 1;
 	if ((d = parse_http_header(e->head, cast_uchar "Transfer-Encoding", NULL))) {
-		if (!strcasecmp(cast_const_char d, "chunked")) {
+		if (!casestrcmp(d, cast_uchar "chunked")) {
 			info->length = -2;
 			info->chunk_remaining = -1;
 		}
@@ -1094,7 +1090,7 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	}
 	if (!info->close && info->length == -1) info->close = 1;
 	if ((d = parse_http_header(e->head, cast_uchar "Last-Modified", NULL))) {
-		if (e->last_modified && strcasecmp(cast_const_char e->last_modified, cast_const_char d)) {
+		if (e->last_modified && casestrcmp(e->last_modified, d)) {
 			delete_entry_content(e);
 			if (c->from) {
 				c->from = 0;
@@ -1126,6 +1122,11 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 		truncate_entry(e, c->from, 0);
 	}
 
+	if (info->https_forward && h == 407) {
+		http_end_request(c, 0, 1, S__OK);
+		return;
+	}
+
 	read_http_data(c, rb);
 }
 
