@@ -140,14 +140,14 @@ static int auth_window(struct object_request *rq, unsigned char *realm)
 		}
 	}
 	urealm = convert(term_charset(term), net_cp, realm, NULL);
-	d = mem_alloc(sizeof(struct dialog) + 5 * sizeof(struct dialog_item) + sizeof(struct auth_dialog) + strlen(cast_const_char _(TEXT_(T_ENTER_USERNAME), term)) + strlen(cast_const_char urealm) + 1 + strlen(cast_const_char _(TEXT_(T_AT), term)) + strlen(cast_const_char host));
+	d = mem_alloc(sizeof(struct dialog) + 5 * sizeof(struct dialog_item) + sizeof(struct auth_dialog) + strlen(cast_const_char get_text_translation(TEXT_(T_ENTER_USERNAME), term)) + strlen(cast_const_char urealm) + 1 + strlen(cast_const_char get_text_translation(TEXT_(T_AT), term)) + strlen(cast_const_char host));
 	memset(d, 0, sizeof(struct dialog) + 5 * sizeof(struct dialog_item) + sizeof(struct auth_dialog));
 	a = (struct auth_dialog *)((unsigned char *)d + sizeof(struct dialog) + 5 * sizeof(struct dialog_item));
-	strcpy(cast_char a->msg, cast_const_char _(TEXT_(T_ENTER_USERNAME), term));
+	strcpy(cast_char a->msg, cast_const_char get_text_translation(TEXT_(T_ENTER_USERNAME), term));
 	strcat(cast_char a->msg, cast_const_char urealm);
 	if (*host) {
 		strcat(cast_char a->msg, "\n");
-		strcat(cast_char a->msg, cast_const_char _(TEXT_(T_AT), term));
+		strcat(cast_char a->msg, cast_const_char get_text_translation(TEXT_(T_AT), term));
 		strcat(cast_char a->msg, cast_const_char host);
 	}
 	mem_free(host);
