@@ -17,6 +17,10 @@
 #include <windows.h>
 #endif
 
+#if defined(__CYGWIN__) && defined(HAVE_LOCALE_H)
+#include <locale.h>
+#endif
+
 #ifdef HAVE_PTHREADS
 #include <pthread.h>
 #endif
@@ -1002,12 +1006,14 @@ void get_path_to_exe(void)
 void init_os_terminal(void)
 {
 #ifdef INTERIX
+	{
 	/* Some sort of terminal bug in Interix, if we run xterm -e links,
 	   terminal doesn't switch to raw mode, executing "stty sane" fixes it.
 	   Don't do this workaround on console. */
-	unsigned char *term = cast_uchar getenv("TERM");
-	if (!term || strncasecmp(cast_const_char term, "interix", 7)) {
-		system("stty sane 2>/dev/null");
+		unsigned char *term = cast_uchar getenv("TERM");
+		if (!term || casecmp(term, cast_uchar "interix", 7)) {
+			system("stty sane 2>/dev/null");
+		}
 	}
 #endif
 #ifdef OS2
@@ -1015,6 +1021,14 @@ void init_os_terminal(void)
 		fatal_exit("Links doesn't work in detached session");
 	}
 #endif
+#if defined(__CYGWIN__) && defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE) && defined(LC_CTYPE)
+	{
+		unsigned char *l = cast_uchar setlocale(LC_CTYPE, "");
+		if (!l || !casestrcmp(l, cast_uchar "C")) {
+			setlocale(LC_CTYPE, "en_US.utf-8");
+		}
+	}
+#endif
 }
 
 #ifdef INTERIX
@@ -1043,8 +1057,8 @@ static inline int is_windows_drive(unsigned char *prog_start, unsigned char *pro
 static inline int is_windows_program(unsigned char *prog_start, unsigned char *prog_end)
 {
 	if (prog_end - prog_start > 4 && (
-		!strncasecmp(cast_const_char(prog_end - 4), ".exe", 4) ||
-		!strncasecmp(cast_const_char(prog_end - 4), ".bat", 4)))
+		!casecmp(prog_end - 4, cast_uchar ".exe", 4) ||
+		!casecmp(prog_end - 4, cast_uchar ".bat", 4)))
 			return 1;
 	return 0;
 }
@@ -1105,6 +1119,7 @@ unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog
 unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog)
 {
 	unsigned char *prog_start, *prog_end;
+	if (!prog) prog = ".exe";
 	cut_program_path(prog, &prog_start, &prog_end);
 	/* Convert path only if the program has ".exe" or ".bat" extension */
 	if (is_windows_program(prog_start, prog_end)) {
@@ -1135,12 +1150,16 @@ unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog
 	return stracpy(file);
 }
 
-#elif defined(DOS)
+#elif defined(OS2) || defined(DOS)
 
 unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog)
 {
 	unsigned char *f, *x;
 	f = stracpy(file);
+#ifdef OS2
+	if (prog)
+		return f;
+#endif
 	x = f;
 	while ((x = cast_uchar strchr(cast_const_char x, '/'))) *x = '\\';
 	return f;
@@ -1244,6 +1263,9 @@ int exe(unsigned char *path, int fg)
 		path = cast_uchar "";
 #endif
 #ifndef EXEC_IN_THREADS
+#ifdef SIGCHLD
+	do_signal(SIGCHLD, SIG_DFL);
+#endif
 	do_signal(SIGPIPE, SIG_DFL);
 #ifdef SIGXFSZ
 	do_signal(SIGXFSZ, SIG_DFL);
@@ -1422,7 +1444,7 @@ int exe(unsigned char *path, int fg)
 	}
 #endif
 	if (!is_winnt()) {
-		sleep(1);
+		portable_sleep(1000);
 		if (ct && GetConsoleTitleA(cast_char buffer2, sizeof buffer2) && !casecmp(buffer2, cast_uchar "start", 5)) {
 			SetConsoleTitleA(cast_const_char buffer);
 		}
@@ -1591,11 +1613,9 @@ int get_windows_cp(int cons)
 void set_window_title(unsigned char *title)
 {
 	unsigned char *t, *p;
-	struct conv_table *ct;
 	if (!title) return;
 	if (is_xterm()) return;
-	ct = get_translation_table(utf8_table, get_windows_cp(1));
-	t = convert_string(ct, title, strlen(cast_const_char title), NULL);
+	t = convert(utf8_table, get_windows_cp(1), title, NULL);
 	for (p = cast_uchar strchr(cast_const_char t, 1); p; p = cast_uchar strchr(cast_const_char(p + 1), 1))
 		*p = ' ';
 	SetConsoleTitleA(cast_const_char t);
@@ -1604,13 +1624,13 @@ void set_window_title(unsigned char *title)
 
 unsigned char *get_window_title(void)
 {
-	struct conv_table *ct;
 	int r;
 	unsigned char buffer[1024];
 	if (is_xterm()) return NULL;
 	if (!(r = GetConsoleTitleA(cast_char buffer, sizeof buffer))) return NULL;
-	ct = get_translation_table(get_windows_cp(1), utf8_table);
-	return convert_string(ct, buffer, r, NULL);
+	if (r >= 1024) r = 1023;
+	buffer[r] = 0;
+	return convert(get_windows_cp(1), utf8_table, buffer, NULL);
 }
 
 static void call_resize(unsigned char *x1, int x, int y)
@@ -1771,7 +1791,6 @@ unsigned char *get_clipboard_text(struct terminal *term)
 #ifdef G
 	if (F && ret) {
 		static int cp = -1;
-		struct conv_table *ct;
 		unsigned char *d;
 		if (cp == -1) {
 			int c = WinQueryCp(os2_hmq);
@@ -1779,8 +1798,7 @@ unsigned char *get_clipboard_text(struct terminal *term)
 			snprintf(cast_char a, sizeof a, "%d", c);
 			if ((cp = get_cp_index(a)) < 0 || cp == utf8_table) cp = 0;
 		}
-		ct = get_translation_table(cp, utf8_table);
-		d = convert_string(ct, ret, strlen(cast_const_char ret), NULL);
+		d = convert(cp, utf8_table, ret, NULL);
 		mem_free(ret);
 		ret = d;
 	}
@@ -1800,7 +1818,6 @@ void set_clipboard_text(struct terminal *term, unsigned char *data)
 #ifdef G
 	if (F) {
 		static int cp = -1;
-		struct conv_table *ct;
 		unsigned char *p;
 		if (cp == -1) {
 			int c = WinQueryCp(os2_hmq);
@@ -1808,8 +1825,7 @@ void set_clipboard_text(struct terminal *term, unsigned char *data)
 			snprintf(cast_char a, sizeof a, "%d", c);
 			if ((cp = get_cp_index(a)) < 0 || cp == utf8_table) cp = 0;
 		}
-		ct = get_translation_table(utf8_table, cp);
-		d = convert_string(ct, data, strlen(cast_const_char data), NULL);
+		d = convert(utf8_table, cp, data, NULL);
 		for (p = cast_uchar strchr(cast_const_char d, 1); p; p = cast_uchar strchr(cast_const_char(p + 1), 1))
 			*p = ' ';
 		data = d;
@@ -2656,6 +2672,15 @@ static void gpm_mouse_in(struct gpm_mouse_spec *gms)
 	if (gev.buttons & GPM_B_LEFT) ev.b = B_LEFT;
 	else if (gev.buttons & GPM_B_MIDDLE) ev.b = B_MIDDLE;
 	else if (gev.buttons & GPM_B_RIGHT) ev.b = B_RIGHT;
+#ifdef GPM_B_FOURTH
+	else if (gev.buttons & GPM_B_FOURTH) ev.b = B_FOURTH;
+#endif
+#ifdef GPM_B_UP
+	else if (gev.buttons & GPM_B_UP) ev.b = B_FIFTH;
+#endif
+#ifdef GPM_B_DOWN
+	else if (gev.buttons & GPM_B_DOWN) ev.b = B_SIXTH;
+#endif
 	else return;
 	if ((int)gev.type & GPM_DOWN) ev.b |= B_DOWN;
 	else if ((int)gev.type & GPM_UP) ev.b |= B_UP;
@@ -3046,7 +3071,7 @@ void os_detach_console(void)
 
 int get_country_language(int c)
 {
-	static const struct {
+	static_const struct {
 		int code;
 		unsigned char *language;
 	} countries[] = {
@@ -3110,7 +3135,7 @@ int get_country_language(int c)
 	if (idx == -1)
 		return -1;
 	for (i = 0; i < n_languages(); i++)
-		if (!strcasecmp(cast_const_char language_name(i), cast_const_char countries[idx].language))
+		if (!casestrcmp(language_name(i), countries[idx].language))
 			return i;
 	return -1;
 }
@@ -3132,6 +3157,34 @@ int os_default_language(void)
 	return -1;
 }
 
+#elif defined(WIN)
+
+int os_default_language(void)
+{
+	LCID id;
+	unsigned char iso639[9];
+	unsigned char iso3166[9];
+	unsigned char loc[8 + 1 + 8 + 1];
+	unsigned char *lang;
+	lang = cast_uchar getenv("LANG");
+	if (lang) {
+		int l = get_language_from_lang(lang);
+		if (l >= 0)
+			return l;
+	}
+	id = GetUserDefaultUILanguage();
+	if (!GetLocaleInfo(id, LOCALE_SISO639LANGNAME, cast_char iso639, 9))
+		return -1;
+	iso3166[0] = 0;
+	GetLocaleInfo(id, LOCALE_SISO3166CTRYNAME, cast_char iso3166, 9);
+	strcpy(cast_char loc, cast_const_char iso639);
+	if (id >= 0x400 && iso3166[0]) {
+		strcat(cast_char loc, "_");
+		strcat(cast_char loc, cast_const_char iso3166);
+	}
+	return get_language_from_lang(loc);
+}
+
 #elif !defined(DOS)
 
 int os_default_language(void)
