@@ -5,7 +5,7 @@
 
 #include "links.h"
 
-static const struct {
+static_const struct {
 	char *prot;
 	int port;
 	void (*func)(struct connection *);
@@ -53,7 +53,7 @@ static int get_prot_info(unsigned char *prot, int *port, void (**func)(struct co
 {
 	int i;
 	for (i = 0; protocols[i].prot; i++)
-		if (!strcasecmp(cast_const_char protocols[i].prot, cast_const_char prot)) {
+		if (!casestrcmp(cast_uchar protocols[i].prot, prot)) {
 			if (port) *port = protocols[i].port;
 			if (func) *func = protocols[i].func;
 			if (nc_func) *nc_func = protocols[i].nc_func;
@@ -282,23 +282,21 @@ static void translate_directories(unsigned char *url)
 		memmove(d, s, strlen(cast_const_char s) + 1);
 		return;
 	}
-	if (dsep(s[0]) && s[1] == '.' && dsep(s[2])) {
-		/**d++ = s[0];*/
-		if (s == dd && !s[3]) goto p;
+	if (dsep(s[0]) && s[1] == '.' && (dsep(s[2]) || !s[2] || end_of_dir(url, s[2]))) {
+		if (!dsep(s[2])) *d++ = *s;
 		s += 2;
 		goto r;
 	}
-	if (dsep(s[0]) && s[1] == '.' && s[2] == '.' && (dsep(s[3]) || !s[3])) {
+	if (dsep(s[0]) && s[1] == '.' && s[2] == '.' && (dsep(s[3]) || !s[3] || end_of_dir(url, s[3]))) {
 		while (d > dd) {
 			d--;
 			if (dsep(*d)) goto b;
 		}
 		b:
-		if (!s[3]) *d++ = *s;
+		if (!dsep(s[3])) *d++ = *s;
 		s += 3;
 		goto r;
 	}
-	p:
 	if ((*d++ = *s++)) goto r;
 }
 
@@ -418,6 +416,8 @@ unsigned char *join_urls(unsigned char *base, unsigned char *rel)
 		n = memacpy(base, s - base);
 		add_to_strn(&n, rel);
 		if (!parse_url(n, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)) goto return_n;
+		add_to_strn(&n, cast_uchar "/");
+		if (!parse_url(n, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)) goto return_n;
 		mem_free(n);
 	}
 	if (!casecmp(cast_uchar "proxy://", rel, 8)) goto prx;
@@ -499,533 +499,15 @@ unsigned char *translate_url(unsigned char *url, unsigned char *cwd)
 #endif
 	if (*ch != ':' || *(url + strcspn(cast_const_char url, "/@")) == '@') {
 		if (*url != '.' && *ch == '.') {
-			unsigned char *f, *e;
-			int i;
+			unsigned char *e, *f, *g;
+			int tl;
 			for (e = ch + 1; *(f = e + strcspn(cast_const_char e, ".:/")) == '.'; e = f + 1)
 				;
-			for (i = 0; i < f - e; i++) if (e[i] < '0' || e[i] > '9') goto noip;
-			goto http;
-			noip:
-			if (f - e == 2 && casecmp(e, cast_uchar "gz", 2)) {
-				http:
-				prefix = cast_uchar "http://", sl = 1;
-			} else {
-				const char * const tld[] = {
-					"com",
-					"edu",
-					"net",
-					"org",
-					"gov",
-					"mil",
-					"int",
-					"arpa",
-					"abogado",
-					"academy",
-					"accountant",
-					"accountants",
-					"active",
-					"actor",
-					"adult",
-					"aero",
-					"agency",
-					"airforce",
-					"allfinanz",
-					"alsace",
-					"amsterdam",
-					"android",
-					"apartments",
-					"app",
-					"aquarelle",
-					"archi",
-					"army",
-					"arpa",
-					"asia",
-					"associates",
-					"attorney",
-					"auction",
-					"audio",
-					"autos",
-					"axa",
-					"band",
-					"bank",
-					"bar",
-					"barclaycard",
-					"barclays",
-					"bargains",
-					"bayern",
-					"beer",
-					"berlin",
-					"best",
-					"bid",
-					"bike",
-					"bingo",
-					"bio",
-					"biz",
-					"black",
-					"blackfriday",
-					"blog",
-					"bloomberg",
-					"blue",
-					"bmw",
-					"bnpparibas",
-					"boo",
-					"boutique",
-					"brussels",
-					"budapest",
-					"build",
-					"builders",
-					"business",
-					"buzz",
-					"bzh",
-					"cab",
-					"cafe",
-					"cal",
-					"camera",
-					"camp",
-					"cancerresearch",
-					"capetown",
-					"capital",
-					"caravan",
-					"cards",
-					"care",
-					"career",
-					"careers",
-					"casa",
-					"cash",
-					"casino",
-					"cat",
-					"catering",
-					"center",
-					"ceo",
-					"cern",
-					"channel",
-					"chat",
-					"cheap",
-					"christmas",
-					"chrome",
-					"church",
-					"citic",
-					"city",
-					"claims",
-					"cleaning",
-					"click",
-					"clinic",
-					"clothing",
-					"club",
-					"coach",
-					"codes",
-					"coffee",
-					"college",
-					"cologne",
-					"com",
-					"community",
-					"company",
-					"computer",
-					"condos",
-					"construction",
-					"consulting",
-					"contractors",
-					"cooking",
-					"cool",
-					"coop",
-					"country",
-					"coupons",
-					"credit",
-					"creditcard",
-					"cricket",
-					"crs",
-					"cruises",
-					"cuisinella",
-					"cymru",
-					"dad",
-					"dance",
-					"date",
-					"dating",
-					"day",
-					"deals",
-					"degree",
-					"delivery",
-					"democrat",
-					"dental",
-					"dentist",
-					"desi",
-					"design",
-					"diamonds",
-					"diet",
-					"digital",
-					"direct",
-					"directory",
-					"discount",
-					"dnp",
-					"dog",
-					"domains",
-					"download",
-					"durban",
-					"dvag",
-					"eat",
-					"edu",
-					"education",
-					"email",
-					"emerck",
-					"energy",
-					"engineer",
-					"engineering",
-					"enterprises",
-					"equipment",
-					"esq",
-					"estate",
-					"eus",
-					"events",
-					"everbank",
-					"exchange",
-					"expert",
-					"exposed",
-					"express",
-					"fail",
-					"faith",
-					"farm",
-					"fashion",
-					"feedback",
-					"finance",
-					"financial",
-					"firmdale",
-					"fish",
-					"fishing",
-					"fit",
-					"fitness",
-					"flights",
-					"florist",
-					"flowers",
-					"flsmidth",
-					"fly",
-					"foo",
-					"football",
-					"forsale",
-					"foundation",
-					"frl",
-					"frogans",
-					"fund",
-					"furniture",
-					"futbol",
-					"fyi",
-					"gal",
-					"gallery",
-					"garden",
-					"gbiz",
-					"gent",
-					"gift",
-					"gifts",
-					"gives",
-					"glass",
-					"gle",
-					"global",
-					"globo",
-					"gmail",
-					"gmo",
-					"gmx",
-					"gold",
-					"golf",
-					"google",
-					"gop",
-					"gov",
-					"graphics",
-					"gratis",
-					"green",
-					"gripe",
-					"guide",
-					"guitars",
-					"guru",
-					"hamburg",
-					"haus",
-					"healthcare",
-					"help",
-					"here",
-					"hiphop",
-					"hiv",
-					"hockey",
-					"holdings",
-					"holiday",
-					"homes",
-					"horse",
-					"host",
-					"hosting",
-					"house",
-					"how",
-					"hsbc",
-					"ibm",
-					"immo",
-					"immobilien",
-					"industries",
-					"info",
-					"ing",
-					"ink",
-					"institute",
-					"insure",
-					"int",
-					"international",
-					"investments",
-					"irish",
-					"jetzt",
-					"jewelry",
-					"jobs",
-					"joburg",
-					"juegos",
-					"kaufen",
-					"kim",
-					"kitchen",
-					"kiwi",
-					"koeln",
-					"krd",
-					"kred",
-					"lacaixa",
-					"land",
-					"lat",
-					"latrobe",
-					"lawyer",
-					"lds",
-					"lease",
-					"legal",
-					"lgbt",
-					"life",
-					"lighting",
-					"limited",
-					"limo",
-					"link",
-					"loan",
-					"loans",
-					"lol",
-					"london",
-					"lotto",
-					"love",
-					"ltda",
-					"luxe",
-					"luxury",
-					"madrid",
-					"maison",
-					"management",
-					"mango",
-					"market",
-					"marketing",
-					"markets",
-					"mba",
-					"media",
-					"meet",
-					"melbourne",
-					"meme",
-					"memorial",
-					"men",
-					"menu",
-					"miami",
-					"mil",
-					"mini",
-					"mobi",
-					"moda",
-					"moe",
-					"monash",
-					"money",
-					"mormon",
-					"mortgage",
-					"moscow",
-					"motorcycles",
-					"mov",
-					"movie",
-					"museum",
-					"nagoya",
-					"name",
-					"nato",
-					"navy",
-					"net",
-					"network",
-					"neustar",
-					"new",
-					"news",
-					"nexus",
-					"ngo",
-					"nhk",
-					"ninja",
-					"nra",
-					"nrw",
-					"nyc",
-					"okinawa",
-					"one",
-					"ong",
-					"onl",
-					"onlajn",
-					"online",
-					"ooo",
-					"org",
-					"organic",
-					"otsuka",
-					"ovh",
-					"paris",
-					"partners",
-					"parts",
-					"party",
-					"pharmacy",
-					"photo",
-					"photography",
-					"photos",
-					"physio",
-					"piaget",
-					"pics",
-					"pictures",
-					"pink",
-					"pizza",
-					"place",
-					"plumbing",
-					"plus",
-					"pohl",
-					"poker",
-					"porn",
-					"post",
-					"praxi",
-					"press",
-					"pro",
-					"prod",
-					"productions",
-					"prof",
-					"properties",
-					"property",
-					"pub",
-					"qpon",
-					"quebec",
-					"racing",
-					"realtor",
-					"recipes",
-					"red",
-					"rehab",
-					"reise",
-					"reisen",
-					"reit",
-					"ren",
-					"rentals",
-					"repair",
-					"report",
-					"republican",
-					"rest",
-					"restaurant",
-					"review",
-					"reviews",
-					"rich",
-					"rio",
-					"rip",
-					"rocks",
-					"rodeo",
-					"rsvp",
-					"ruhr",
-					"run",
-					"ryukyu",
-					"saarland",
-					"sajt",
-					"sale",
-					"sarl",
-					"sca",
-					"scb",
-					"schmidt",
-					"school",
-					"schule",
-					"science",
-					"scot",
-					"services",
-					"sexy",
-					"shiksha",
-					"shoes",
-					"show",
-					"singles",
-					"site",
-					"soccer",
-					"social",
-					"software",
-					"sohu",
-					"solar",
-					"solutions",
-					"soy",
-					"space",
-					"spiegel",
-					"style",
-					"sucks",
-					"supplies",
-					"supply",
-					"support",
-					"surf",
-					"surgery",
-					"suzuki",
-					"sydney",
-					"systems",
-					"taipei",
-					"tatar",
-					"tattoo",
-					"tax",
-					"taxi",
-					"team",
-					"technology",
-					"tel",
-					"tennis",
-					"theater",
-					"tienda",
-					"tips",
-					"tires",
-					"tirol",
-					"today",
-					"tokyo",
-					"tools",
-					"top",
-					"tours",
-					"town",
-					"toys",
-					"trade",
-					"training",
-					"travel",
-					"tui",
-					"university",
-					"uno",
-					"uol",
-					"vacations",
-					"vegas",
-					"ventures",
-					"versicherung",
-					"vet",
-					"viajes",
-					"video",
-					"villas",
-					"vision",
-					"vlaanderen",
-					"vodka",
-					"vote",
-					"voting",
-					"voto",
-					"voyage",
-					"wales",
-					"wang",
-					"watch",
-					"webcam",
-					"website",
-					"wed",
-					"wedding",
-					"whoswho",
-					"wien",
-					"wiki",
-					"williamhill",
-					"win",
-					"wme",
-					"work",
-					"works",
-					"world",
-					"wtc",
-					"wtf",
-					"xxx",
-					"xyz",
-					"yandex",
-					"yoga",
-					"yokohama",
-					"youtube",
-					"zone",
-					"zuerich",
-					"onion",
-					"exit",
-					NULL };
-				for (i = 0; tld[i]; i++) if ((size_t)(f - e) == strlen(tld[i]) && !casecmp(cast_uchar tld[i], e, f - e)) goto http;
-			}
+			g = memacpy(e, f - e);
+			tl = is_tld(g);
+			mem_free(g);
+			if (tl)
+				http: prefix = cast_uchar "http://", sl = 1;
 		}
 		if (*ch == '@' || *ch == ':' || !cmpbeg(url, cast_uchar "ftp.")) prefix = cast_uchar "ftp://", sl = 1;
 		goto set_prefix;
@@ -1117,3 +599,11 @@ void add_conv_str(unsigned char **s, int *l, unsigned char *b, int ll, int encod
 	}
 }
 
+unsigned char *display_url(struct terminal *term, unsigned char *url)
+{
+	unsigned char *u, *uu;
+	if (!url) return stracpy(cast_uchar "");
+	u = stracpy(url);
+	if ((uu = cast_uchar strchr(cast_const_char u, POST_CHAR))) *uu = 0;
+	return u;
+}
