@@ -12,6 +12,11 @@
 #include <lib$routines.h>
 #include <libclidef.h>
 
+#if defined(HAVE_OPENSSL) && !defined(OPENSSL_NO_SHA) && !defined(OPENSSL_NO_SHA1)
+#include <openssl/sha.h>
+#define USE_SHA
+#endif
+
 #ifdef __VAX
 struct _iosb {
 	unsigned short iosb$w_status;
@@ -41,7 +46,7 @@ struct _generic_64 {
 
 #define VIRTUAL_PIPE_SIZE		512
 
-#define GETTIMEOFDAY_POOL		16
+#define GETTIMEOFDAY_POOL		256
 
 /*#define TEST_WAKE_BUG*/
 /*#define TRACE_PIPES*/
@@ -128,10 +133,8 @@ static volatile int must_redraw = 0;
 static volatile int ctrl_c = 0;
 static volatile int ctrl_y = 0;
 
-#ifdef __VAX
 static volatile struct timeval gettimeofday_pool[GETTIMEOFDAY_POOL];
 static volatile unsigned gettimeofday_clock = 0;
-#endif
 
 static void vms_key_handler(int mode, void (*ast)(void))
 {
@@ -318,7 +321,7 @@ int get_input_handle(void)
 		vms_fatal_exit("pthread_mutex_lock failed: %d", ret);
 	lib$disable_ctrl(&disable_ctrl_mask, &old_ctrl_mask);
 	vms_thread_high_priority = 1;
-	input_handle = start_thread(vms_input_thread, NULL, 0);
+	input_handle = start_thread(vms_input_thread, NULL, 0, 0);
 	if (input_handle == -1)
 		vms_fatal_exit("unable to start keyboard thread");
 	vms_thread_high_priority = 0;
@@ -352,7 +355,7 @@ int vms_x11_fd(int ef)
 		return x11_pipe;
 	}
 	x11_ef = ef;
-	x11_pipe = start_thread(vms_x11_thread, NULL, 0);
+	x11_pipe = start_thread(vms_x11_thread, NULL, 0, 0);
 	if (x11_pipe == -1)
 		vms_fatal_exit("unable to start Xwindow event thread");
 	set_nonblock(x11_pipe);
@@ -377,10 +380,10 @@ static int vms_socketpair(int *fd)
 	struct sockaddr_in sa1, sa2;
 	int retry_count = 0;
 	again:
-	EINTRLOOP(s1, socket(PF_INET, SOCK_STREAM, IPPROTO_TCP));
+	s1 = c_socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (s1 < 0)
 		goto err0;
-	EINTRLOOP(s2, socket(PF_INET, SOCK_STREAM, IPPROTO_TCP));
+	s2 = c_socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (s2 < 0)
 		goto err1;
 	memset(&sa1, 0, sizeof(sa1));
@@ -405,7 +408,7 @@ static int vms_socketpair(int *fd)
 	if (rs)
 		goto err2;
 	l = sizeof(sa2);
-	EINTRLOOP(s3, accept(s1, (struct sockaddr *)&sa2, &l));
+	s3 = c_accept(s1, (struct sockaddr *)&sa2, &l);
 	if (s3 < 0)
 		goto err2;
 	if (sa1.sin_addr.s_addr != sa2.sin_addr.s_addr ||
@@ -510,15 +513,13 @@ static void get_abstime(time_t sec, unsigned usec, struct timespec *ts)
 		tv.tv_usec -= 1000000;
 		tv.tv_sec++;
 	}
-#ifdef __VAX
 	{
 		unsigned c = gettimeofday_clock;
 		if (c >= GETTIMEOFDAY_POOL) c = 0;
 		gettimeofday_clock = c;
-		gettimeofday_pool[c].tv_sec += tv.tv_sec;
-		gettimeofday_pool[c].tv_usec += tv.tv_usec;
+		gettimeofday_pool[c].tv_sec = (unsigned long)gettimeofday_pool[c].tv_sec * 11 + (unsigned long)tv.tv_sec;
+		gettimeofday_pool[c].tv_usec = (unsigned long)gettimeofday_pool[c].tv_usec * 11 + (unsigned long)tv.tv_usec;
 	}
-#endif
 	ts->tv_sec = tv.tv_sec;
 	ts->tv_nsec = tv.tv_usec * 1000;
 #endif
@@ -595,7 +596,7 @@ static void vms_select_thread(void *p, int n)
 			}
 			if (FD_ISSET(i, &remove_set)) {
 				FD_CLR(i, &remove_set);
-#ifndef  TEST_WAKE_BUG
+#ifndef TEST_WAKE_BUG
 				FD_CLR(i, &select_set_read);
 				FD_CLR(i, &select_set_write);
 				FD_CLR(i, &select_set_exception);
@@ -846,7 +847,7 @@ int vms_select(int n, fd_set *rs, fd_set *ws, fd_set *es, struct timeval *t)
 				increase_dynamic_time();
 			/*fprintf(stderr, "wait time: %d\n", wait_time);*/
 			get_abstime(0, wait_time, &ts2);
-			if (t && 
+			if (t &&
 			    (ts2.tv_sec > ts.tv_sec ||
 			    (ts2.tv_sec == ts.tv_sec && ts2.tv_nsec > ts.tv_nsec)))
 				goto full_wait;
@@ -1006,11 +1007,15 @@ int vms_write(int fd, const void *buf, size_t size)
 
 void init_os(void)
 {
+#if !defined(__VAX) && defined(__FEATURE_MODE_INIT_STATE) && defined(__FEATURE_MODE_CURVAL)
+	if (decc$feature_get("DECC$EFS_CHARSET", __FEATURE_MODE_INIT_STATE) < 1)
+		decc$feature_set("DECC$EFS_CHARSET", __FEATURE_MODE_CURVAL, 1);
+#endif
 #if defined(OPENVMS_64BIT)
 	{
 		int i;
 		unsigned char **new_argv = malloc((g_argc + 1) * sizeof(unsigned char *));
-		if (!new_argv) fatal_exit("can't allocate argv");
+		if (!new_argv) vms_fatal_exit("can't allocate argv");
 		for (i = 0; i <= g_argc; i++)
 			new_argv[i] = (unsigned char *)(my_intptr_t)(((unsigned *)g_argv)[i]);
 		g_argv = new_argv;
@@ -1045,7 +1050,7 @@ void init_os(void)
 		add_to_thread_set(select_thread_signal[0], &select_set_read);
 		select_set_max = select_thread_signal[0] + 1;
 		vms_thread_high_priority = -1;
-		ret = start_thread(vms_select_thread, NULL, 0);
+		ret = start_thread(vms_select_thread, NULL, 0, 0);
 		if (ret == -1)
 			vms_fatal_exit("unable to start select thread");
 		vms_thread_high_priority = 0;
@@ -1054,19 +1059,77 @@ void init_os(void)
 #endif
 }
 
-void os_seed_random(unsigned char **pool, unsigned *pool_size)
+void os_seed_random(unsigned char **pool, int *pool_size)
 {
+	struct history_item *hi;
+	DIR *dir;
+	int n, h;
+
+	*pool = init_str();
+	*pool_size = 0;
+
 	/*
 	 * This is not very secure, but I don't know a better way.
-	 * It is needed only on VAX; Alpha has a better random number seeding.
 	 */
-#ifdef __VAX
-	*pool = memacpy((unsigned char *)(void *)&gettimeofday_pool, sizeof gettimeofday_pool);
-	*pool_size = sizeof gettimeofday_pool;
-#else
-	*pool = DUMMY;
-	*pool_size = 0;
+	add_bytes_to_str(pool, pool_size, (unsigned char *)(void *)&gettimeofday_pool, sizeof gettimeofday_pool);
+
+#ifdef USE_SHA
+	/*
+	 * Make sure that even if the transformation is reversible (due to
+	 * poor randomness on OpenVMS), the adversary won't be able to find
+	 * any URLs in the history.
+	 */
+	n = 0;
+	foreach(hi, goto_url_history.items) {
+		SHA_CTX ctx;
+		unsigned char result[SHA_DIGEST_LENGTH];
+		unsigned char sum;
+		int i;
+
+		SHA1_Init(&ctx);
+		SHA1_Update(&ctx, hi->d, strlen(cast_const_char hi->d));
+		SHA1_Final(result, &ctx);
+
+		sum = 0;
+		for (i = 0; i < SHA_DIGEST_LENGTH; i++) {
+			sum += result[i] + (result[i] >> 4);
+		}
+		add_chr_to_str(pool, pool_size, sum & 0xf);
+		if (++n >= 64)
+			break;
+	}
 #endif
+
+	dir = c_opendir(cast_uchar "/SYS$LOGIN");
+	if (dir) {
+		for (n = 0; n < 256; n++) {
+			struct dirent *de;
+			unsigned char *path;
+			struct stat st;
+			ENULLLOOP(de, (void *)readdir(dir));
+			if (!de)
+				break;
+			path = stracpy(cast_uchar "/SYS$LOGIN/");
+			add_to_strn(&path, cast_uchar de->d_name);
+			if (!stat(cast_const_char path, &st)) {
+				add_bytes_to_str(pool, pool_size, (unsigned char *)&st.st_ctime, (int)sizeof st.st_ctime);
+			}
+			mem_free(path);
+		}
+		closedir(dir);
+	}
+
+	h = c_open(cast_uchar "/SYS$LOGIN/SSH2/RANDOM_SEED", O_RDONLY);
+	if (h == -1)
+		h = c_open(cast_uchar "/SYS$LOGIN/SSH/RANDOM_SEED", O_RDONLY);
+	if (h != -1) {
+		unsigned char buffer[512];
+		int r;
+		r = hard_read(h, buffer, (int)sizeof buffer);
+		if (r >= 0)
+			add_bytes_to_str(pool, pool_size, buffer, r);
+		EINTRLOOP(r, close(h));
+	}
 }
 
 void terminate_osdep(void)
